;+
; NAME:
;       IAA_CONTOUR
;
; PURPOSE:
;
;       This program is a simple wrapper for the IDL CONTOUR command.
;
; AUTHOR:
;
;       Jos Enrique Ruiz
;       Inspired upon David Fanning's MPI_PLOT
;
; CATEGORY:
;
;       Graphics
;
; CALLING SEQUENCE:
;
;       IAA_Contour, z, x, y
;
; INPUT PARAMETERS:
;
;       z - The dependent data. z(x,y)
;
;       x - The independent data.
;
;       y - The independent data.
;
;
; INPUT KEYWORDS:
;
;       BACKGROUND - The name of a background color. (See below for a list of color names.)
;         By default on 24-bit systems: 'IVORY'. Uses 'GRAY' on 8-bit systems.
;
;       COLOR - The name of the axis color
;
;       NLEVELS - Number of contour levels.
;
;       TITLE - The title of the contour. By default, a null string.
;
;       XLOG = Set this keyword to use logarithmic axis styling on the X axis.
;
;       XTITLE - The title of the X axis of the contour. By default, a null string.
;
;       YLOG = Set this keyword to use logarithmic axis styling on the Y axis.
;
;       YTITLE - The title of the Y axis of the contour. By default, a null string.
;
;       In addition, any keyword appropriate for the IAA_CONTOURCONFIG object program can be used.
;       Among those keywords, are these most popular ones:
;
;       CHARSIZE - The character size of the contour. By default, 1.0.
;
;       CHARTHICK - The character thickness of the contour. By default, 1.0.
;
;       FONT - The type of contour font: -1=Hershey, 0=Hardware, 1=True-Type. By default, !P.FONT.
;
;       C_LINESTYLE - The contour linestyle. By default, 0. Possible values are:
;         0 - Solid Line
;         1 - Dotted
;         2 - Dashed
;         3 - Dash Dot
;         4 - Dash Dot Dot
;         5 - Long Dash
;.
;       POSITION - The position of the contour in the contour window in normalized coordinates. By default, [0.20, 0.15, 0.95, 0.95].
;
;       THICK - The contour line thickness. By default, 1.0.
;
;       TICKLEN - The contour tick length. By default, 0.02.
;
; COLOR NAMES:
;
;        The following color names can be used for BACKGROUND, COLOR and DATACOLOR keywords:
;
;           White, Snow, Ivory, Light Yellow, Cornsilk, Beige, Seashell, Linen, Antique White,
;           Papaya, Almond, Bisque, Moccasin, Wheat, Burlywood, Tan, Light Gray, Lavender,
;           Medium Gray, Gray, Slate Gray, Dark Gray , Charcoal, Black, Light Cyan, Powder Blue,
;           Sky Blue, Steel Blue, Dodger Blue, Royal Blue, Blue, Navy, Honeydew, Pale Green,
;           Aquamarine, Spring Green, Cyan, Turquoise, Sea Green, Forest Green, Green Yellow,
;           Chartreuse, Lawn Green, Green, Lime Green, Olive Drab, Olive, Dark Green, Pale Goldenrod,
;           Khaki, Dark Khaki, Yellow, Gold, Goldenrod, Dark Goldenrod, Saddle Brown, Rose,
;           Pink, Rosy Brown, Sandy Brown, Peru, Indian Red, Chocolate, Sienna, Dark Salmon,
;           Salmon, Light Salmon, Orange, Coral, Light Coral, Firebrick, Brown, Hot Pink,
;           Deep Pink, Magenta, Tomato, Orange Red, Red, Violet Red, Maroon, Thistle, Plum,
;           Violet, Orchid, Medium Orchid, Dark Orchid, Blue Violet, and Purple.
;
;
; RESTRICTIONS
;
;       Colors will be loaded in the color table.
;
; EXAMPLE:
;
;
;       IAA_CONTOUR, z, x, y
;
;


PRO IAA_Contour_about, event

value = [$
'', $
'IAA Contour Version 1.0', $
'', $
'Jos Enrique Ruiz', $
'Dept. del Sistema Solar', $
'Instituto Astrofsica de Andaluca', $
'CSIC', $
'' $
]

about = DIALOG_MESSAGE(value, /information)
END ;---------------------------------------------------------------------------


PRO IAA_Contour_DrawTheContour, infoptr

; This procedure draws the contour on the output device.

   ; Error handling.

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /cancel
   ok = Error_Message(/Traceback)
ENDIF

   ; Set current graphics window for devices that support it.

IF (!D.Flags AND 256) NE 0 THEN WSet, (*infoptr).wid

   ; Draw the contour.

contourkeywords = (*infoptr).contourconfig->GetKeywords()

   ; Define levels

IF contourkeywords.cb_log EQ 1 THEN BEGIN


   nlevels = (CEIL(ALOG10(contourkeywords.barmaxval))-FLOOR(ALOG10(contourkeywords.barminval)))+1
   (*infoptr).contourconfig->SetProperty, nlevels=nlevels
   LevelsArr = FltArr(nlevels)

   step = 10.^CEIL(ALOG10(contourkeywords.barmaxval))
   FOR i=0,nlevels-1 DO BEGIN
   LevelsArr[i]=step
   step = step/10
   ENDFOR

   LevelsArr = Reverse(LevelsArr)

ENDIF ELSE BEGIN

   LevelsArr = FltArr(contourkeywords.nlevels)

   step = (contourkeywords.barmaxval - contourkeywords.barminval)/(contourkeywords.nlevels*1.)
   FOR i=0,contourkeywords.nlevels-1 DO BEGIN
   LevelsArr[i]=contourkeywords.barminval+i*step
   ENDFOR

ENDELSE

    ; Calculate the contour colors.

step = !D.Table_Size / Float(contourkeywords.nlevels)
contourColors = Indgen(contourkeywords.nlevels) * step

IF (!D.Flags AND 256) EQ 0 THEN BEGIN
      PolyFill, [0, 0, 1, 1, 0], [0, 1, 1, 0, 0], /Normal, Color=contourkeywords.background
      contourkeywords.NOERASE=1
ENDIF

IF contourkeywords.FILL EQ 0 THEN BEGIN

    Contour, (*infoptr).zdata, (*infoptr).xdata, (*infoptr).ydata, C_COLORS=contourColors, $
    Min_value=contourkeywords.barminval, Max_value=contourkeywords.barmaxval, Levels=LevelsArr, _Extra=contourkeywords

ENDIF ELSE BEGIN

    IF contourkeywords.OVERPLOT GT 0 THEN BEGIN
       contourkeywords.OVERPLOT=0
       Contour, (*infoptr).zdata, (*infoptr).xdata, (*infoptr).ydata, Min_value=contourkeywords.barminval, /closed, /cell_fill, $
       Max_value=contourkeywords.barmaxval, Levels=LevelsArr, _Extra=contourkeywords
       contourkeywords.FILL=0
       contourkeywords.OVERPLOT=1
       Contour, (*infoptr).zdata, (*infoptr).xdata, (*infoptr).ydata, C_COLORS=contourkeywords.color, $
       Min_value=contourkeywords.barminval, Levels=LevelsArr, Max_value=contourkeywords.barmaxval, _Extra=contourkeywords
       contourkeywords.FILL=1
    ENDIF ELSE BEGIN
        Contour, (*infoptr).zdata, (*infoptr).xdata, (*infoptr).ydata, /closed, /cell_fill, $
        Min_value=contourkeywords.barminval, Levels=LevelsArr, Max_value=contourkeywords.barmaxval, _Extra=contourkeywords
    ENDELSE

ENDELSE


colorbarposition = contourkeywords.position
colorbarposition[0] = colorbarposition[2]+0.05
colorbarposition[2] = colorbarposition[0]+0.05

VCOLORBAR, BOTTOM=3, NCOLORS=!D.Table_Size-7, POSITION=colorbarposition, /VERTICAL, /RIGHT, CHARSIZE=contourkeywords.CHARSIZE, $
      CHARTHICK=contourkeywords.CHARTHICK, FONT=contourkeywords.FONT, TICKLEN = contourkeywords.TICKLEN*8, $
      DIVISIONS=contourkeywords.BARDIVISIONS, MINOR=contourkeywords.BARMINOR, FORMAT=contourkeywords.BARFORMAT, $
      TITLE=contourkeywords.BARTITLE, MINRANGE=contourkeywords.BARMINVAL, MAXRANGE=contourkeywords.BARMAXVAL, $
      CB_LOG=contourkeywords.CB_LOG, INVERTCOLORS=contourkeywords.INVERTCOLORS, COLOR=contourkeywords.color



END; -------------------------------------------------------------------------


PRO IAA_Contour_PostScript, event

; This event handler creates PostScript output.

   ; Get info structure.

Widget_Control, event.top, Get_UValue=infoptr

   ; Make the display window current.

WSet, (*infoptr).wid

keywords = PSWindow()
pskeywords = PSConfig(Cancel=cancelled, _Extra=keywords, Group_Leader=event.top)
IF cancelled THEN RETURN

   ; Configure the PostScript device.

thisDevice = !D.Name
Set_Plot, 'PS'
Device, _Extra=pskeywords

   ; Display the grapics. Background color will have to be done
   ; separately.

contourkeywords = (*infoptr).contourconfig->GetKeywords()

  ; Define levels

LevelsArr = FltArr(contourkeywords.nlevels)

IF contourkeywords.cb_log EQ 1 THEN BEGIN

   step = 10.^CEIL(ALOG10(contourkeywords.barmaxval))
   FOR i=0,contourkeywords.nlevels-1 DO BEGIN
   LevelsArr[i]=step
   step = step/10
   ENDFOR

   LevelsArr = Reverse(LevelsArr)

ENDIF ELSE BEGIN

   step = (contourkeywords.barmaxval - contourkeywords.barminval)/(contourkeywords.nlevels*1.)
   FOR i=0,contourkeywords.nlevels-1 DO BEGIN
   LevelsArr[i]=contourkeywords.barminval+i*step
   ENDFOR

ENDELSE

    ; Calculate the contour colors.

step = !D.Table_Size / Float(contourkeywords.nlevels)
contourColors = Indgen(contourkeywords.nlevels) * step

IF pskeywords.color EQ 1 THEN BEGIN

   PolyFill, [0, 0, 1, 1, 0], [0, 1, 1, 0, 0], /Normal, Color=!P.Color
   contourkeywords.NOERASE=1

ENDIF ELSE BEGIN

   contourkeywords.color = !P.Color
   contourkeywords.background = !P.Background

ENDELSE

IF contourkeywords.FILL EQ 0 THEN BEGIN

    Contour, (*infoptr).zdata, (*infoptr).xdata, (*infoptr).ydata, C_COLORS=contourColors, $
    Min_value=contourkeywords.barminval, Max_value=contourkeywords.barmaxval, Levels=LevelsArr, _Extra=contourkeywords

ENDIF ELSE BEGIN

    IF contourkeywords.OVERPLOT GT 0 THEN BEGIN
       contourkeywords.OVERPLOT=0
       Contour, (*infoptr).zdata, (*infoptr).xdata, (*infoptr).ydata, $
       Min_value=contourkeywords.barminval, Max_value=contourkeywords.barmaxval, /closed, /cell_fill, _Extra=contourkeywords
       contourkeywords.FILL=0
       contourkeywords.OVERPLOT=1
       Contour, (*infoptr).zdata, (*infoptr).xdata, (*infoptr).ydata, C_COLORS=contourkeywords.color, $
       Min_value=contourkeywords.barminval, Max_value=contourkeywords.barmaxval, _Extra=contourkeywords
       contourkeywords.FILL=1
    ENDIF ELSE BEGIN
        Contour, (*infoptr).zdata, (*infoptr).xdata, (*infoptr).ydata, $
        Min_value=contourkeywords.barminval, Max_value=contourkeywords.barmaxval, /closed, /cell_fill, _Extra=contourkeywords
    ENDELSE

ENDELSE

colorbarposition = contourkeywords.position
colorbarposition[0] = colorbarposition[2]+0.05
colorbarposition[2] = colorbarposition[0]+0.05

VCOLORBAR, BOTTOM=3, NCOLORS=!D.Table_Size-7, POSITION=colorbarposition, /VERTICAL, /RIGHT, CHARSIZE=contourkeywords.CHARSIZE, $
      CHARTHICK=contourkeywords.CHARTHICK, FONT=contourkeywords.FONT, TICKLEN = contourkeywords.TICKLEN*8, $
      DIVISIONS=contourkeywords.BARDIVISIONS, MINOR=contourkeywords.BARMINOR, FORMAT=contourkeywords.BARFORMAT, $
      TITLE=contourkeywords.BARTITLE, MINRANGE=contourkeywords.BARMINVAL, MAXRANGE=contourkeywords.BARMAXVAL, $
      CB_LOG=contourkeywords.CB_LOG, INVERTCOLORS=contourkeywords.INVERTCOLORS, COLOR=contourkeywords.color

    ; Close the PostScript file and clean up.

Device, /Close_File
Set_Plot, thisDevice

END; -------------------------------------------------------------------------



PRO IAA_Contour_Print, event

; This event handler executes the command in the Printer device.

   ; Set up the printer.

ok = Dialog_PrinterSetup()
IF NOT ok THEN RETURN

   ; Get info structure and printer orientation.

Widget_Control, event.top, Get_UValue=infoptr
Widget_Control, event.id, Get_UValue=orientation

   ; Save the current graphics device.

thisDevice = !D.Name
obackground = !P.Background
ocolor = !P.Color

   ; Make the display window current.

WSet, (*infoptr).wid

   ; With printers, you have to load your drawing colors
   ; *before* you enter the PRINTER device and then copy
   ; them into the device. It's a pain in the ol' wazoo,
   ; but there is no help for it. :-(

contourkeywords = (*infoptr).contourconfig->GetKeywords()


TVLCT, FSC_Color(contourkeywords.name_background, /Triple), contourkeywords.background
TVLCT, FSC_Color(contourkeywords.name_color, /Triple), contourkeywords.color

   ; Set up the printer. You may have to adjust the fudge factors
   ; to account for the printable area offset.

CASE orientation OF
   'PORTRAIT': BEGIN
      keywords = PSWindow(/Printer, Fudge=0.25)
      Set_Plot, 'PRINTER', /Copy
      Device, Portrait=1
      ENDCASE
   'LANDSCAPE': BEGIN
      keywords = PSWindow(/Printer, /Landscape, Fudge=0.25)
      Set_Plot, 'PRINTER', /Copy
      Device, Landscape=1
      ENDCASE
ENDCASE
Device, _Extra=keywords

   ; Display the graphics.

!P.Background = obackground
!P.Color = ocolor

IAA_Contour_DrawTheContour, infoptr

   ; Close the printer file and clean up.

Device, /Close_Document
Set_Plot, thisDevice



END ;----------------------------------------------------------------------------------------



PRO IAA_Contour_SaveAs, event

; Saves the current display window as output files.

   ; Get the info structure and the appropriate file extension.

Widget_Control, event.top, Get_UValue=infoptr
Widget_Control, event.id, Get_UValue=file_extension

   ; Base name for file output.

basename = 'iaa_contour'

   ; Take a snapshot of the display window and write the file.

WSet, (*infoptr).wid
CASE file_extension OF
   'BMP'  : image = TVREAD(Filename = basename, /BMP)
   'GIF'  : image = TVREAD(Filename = basename, /GIF)
   'PICT' : image = TVREAD(Filename = basename, /PICT)
   'JPG'  : image = TVREAD(Filename = basename, /JPEG)
   'TIF'  : image = TVREAD(Filename = basename, /TIFF)
   'PNG'  : image = TVREAD(Filename = basename, /PNG)
ENDCASE

END ;----------------------------------------------------------------------------------------



PRO IAA_Contour_Quit, event
Widget_Control, event.top, /Destroy
END ;-----------------------------------------------------------------------------

PRO IAA_Contour_Restore, event

Widget_Control, event.top, Get_UValue=infoptr

filename = Filepath(Root_Dir=ProgramRootDir(), Subdirectory=['savfiles'], (*infoptr).functionality+'.sav')
IAA_Contour_Load, event, filename


END ;-----------------------------------------------------------------------------


PRO IAA_Contour_Cleanup, tlb

   ; This is the cleanup routine for the program. All
   ; created objects and pointers must be destroyed.

Widget_Control, tlb, Get_UValue=infoptr

Obj_Destroy, (*infoptr).xaxis
Obj_Destroy, (*infoptr).yaxis
Obj_Destroy, (*infoptr).contourconfig
Ptr_Free, infoptr
END ;-----------------------------------------------------------------------------



PRO IAA_Contour_Configure_Button, event

   ; This event handler responds to requests to configure the
   ; contour parameters. Events will be sent to the buttonbase widget.

Widget_Control, event.top, Get_UValue=infoptr

   ; Call the GUI method on the Contour Configuration object.

Widget_Control, event.top, Update=0
(*infoptr).contourconfig->GUI, Group_Leader=event.top, All_Events=1, $
   NotifyID=[(*infoptr).buttonbase, event.top]
Widget_Control, event.top, Update=1

END ;-----------------------------------------------------------------------------

PRO IAA_Contour_Load, event, filename

   ; This event handler responds to requests to load contour settings.

    Widget_Control, event.top, Get_UValue=infoptr

    IF N_Elements(filename) EQ 0 THEN $
    filename = DIALOG_PICKFILE(/READ, FILE = '*.sav')

    contourkeywords = (*infoptr).contourconfig->GetKeywords()
    TitleNow = contourkeywords.TITLE
    BarTitleNow = contourkeywords.BARTITLE
    XTitleNow = contourkeywords.XTITLE
    YTitleNow = contourkeywords.YTITLE
    IrregularNow = contourkeywords.IRREGULAR


   ; Restore Contour Settings
    ok = FILE_SEARCH(filename, count=exists)
    IF filename NE '' AND exists GT 0 THEN BEGIN

    restore, filename

    r = congrid(r, 256, !d.table_size)
    g = congrid(g, 256, !d.table_size)
    b = congrid(b, 256, !d.table_size)
    TVLCT, r, g, b

    IF TitleNow EQ '' THEN BEGIN
    (*infoptr).contourconfig->SetProperty, Title=contourkeywords.TITLE
    ENDIF ELSE BEGIN
    (*infoptr).contourconfig->SetProperty, Title=TitleNow
    ENDELSE

    (*infoptr).contourconfig->SetProperty, Nlevels=contourkeywords.NLEVELS
    (*infoptr).contourconfig->SetProperty, Background=contourkeywords.NAME_BACKGROUND
    (*infoptr).contourconfig->SetProperty, Charsize=contourkeywords.CHARSIZE
    (*infoptr).contourconfig->SetProperty, Charthick=contourkeywords.CHARTHICK
    (*infoptr).contourconfig->SetProperty, Color=contourkeywords.NAME_COLOR
    (*infoptr).contourconfig->SetProperty, Font=contourkeywords.FONT
    (*infoptr).contourconfig->SetProperty, Position=contourkeywords.POSITION
    (*infoptr).contourconfig->SetProperty, Subtitle=contourkeywords.SUBTITLE
    (*infoptr).contourconfig->SetProperty, C_linestyle=contourkeywords.C_LINESTYLE
    (*infoptr).contourconfig->SetProperty, Thick=contourkeywords.THICK
    (*infoptr).contourconfig->SetProperty, Ticklen=contourkeywords.TICKLEN
    (*infoptr).contourconfig->SetProperty, Fill=contourkeywords.FILL
    (*infoptr).contourconfig->SetProperty, Follow=contourkeywords.FOLLOW
    (*infoptr).contourconfig->SetProperty, Overplot=contourkeywords.OVERPLOT
    (*infoptr).contourconfig->SetProperty, Noerase=contourkeywords.NOERASE

    IF BarTitleNow EQ '' THEN BEGIN
    (*infoptr).contourconfig->SetProperty, BarTitle=contourkeywords.BARTITLE
    ENDIF ELSE BEGIN
    (*infoptr).contourconfig->SetProperty, BarTitle=BarTitleNow
    ENDELSE

    (*infoptr).contourconfig->SetProperty, Bardivisions=contourkeywords.BARDIVISIONS
    (*infoptr).contourconfig->SetProperty, Barminor=contourkeywords.BARMINOR
    (*infoptr).contourconfig->SetProperty, Barformat=contourkeywords.BARFORMAT
    (*infoptr).contourconfig->SetProperty, Invertcolors=contourkeywords.INVERTCOLORS
    (*infoptr).contourconfig->SetProperty, Cb_Log=contourkeywords.CB_LOG

    (*infoptr).contourconfig->SetProperty, Irregular=IrregularNow
    (*infoptr).contourconfig->SetProperty, Isotropic=contourkeywords.ISOTROPIC
    (*infoptr).contourconfig->SetProperty, Downhill=contourkeywords.DOWNHILL

    IF XTitleNow EQ '' THEN BEGIN
    (*infoptr).xaxis->SetProperty, Title=contourkeywords.XTITLE, /xaxis
    ENDIF ELSE BEGIN
    (*infoptr).xaxis->SetProperty, Title=XTitleNow
    ENDELSE

    (*infoptr).xaxis->SetProperty, Charsize=contourkeywords.XCHARSIZE, /xaxis
    (*infoptr).xaxis->SetProperty, Gridstyle=contourkeywords.XGRIDSTYLE, /xaxis
    (*infoptr).xaxis->SetProperty, Minor=contourkeywords.XMINOR, /xaxis
    (*infoptr).xaxis->SetProperty, Style=contourkeywords.XSTYLE, /xaxis
    (*infoptr).xaxis->SetProperty, Thick=contourkeywords.XTHICK, /xaxis
    (*infoptr).xaxis->SetProperty, Tickformat=contourkeywords.XTICKFORMAT, /xaxis
    (*infoptr).xaxis->SetProperty, Ticklen=contourkeywords.XTICKLEN, /xaxis
    (*infoptr).xaxis->SetProperty, Ticks=contourkeywords.XTICKS, /xaxis
    (*infoptr).xaxis->SetProperty, Range=contourkeywords.XRANGE, /xaxis
    (*infoptr).xaxis->SetProperty, Log=contourkeywords.XLOG, /xaxis

    IF YTitleNow EQ '' THEN BEGIN
    (*infoptr).yaxis->SetProperty, Title=contourkeywords.YTITLE, /yaxis
    ENDIF ELSE BEGIN
    (*infoptr).yaxis->SetProperty, Title=YTitleNow
    ENDELSE

    (*infoptr).yaxis->SetProperty, Charsize=contourkeywords.YCHARSIZE, /yaxis
    (*infoptr).yaxis->SetProperty, Gridstyle=contourkeywords.YGRIDSTYLE, /yaxis
    (*infoptr).yaxis->SetProperty, Minor=contourkeywords.YMINOR, /yaxis
    (*infoptr).yaxis->SetProperty, Style=contourkeywords.YSTYLE, /yaxis
    (*infoptr).yaxis->SetProperty, Thick=contourkeywords.YTHICK, /yaxis
    (*infoptr).yaxis->SetProperty, Tickformat=contourkeywords.YTICKFORMAT, /yaxis
    (*infoptr).yaxis->SetProperty, Ticklen=contourkeywords.YTICKLEN, /yaxis
    (*infoptr).yaxis->SetProperty, Ticks=contourkeywords.YTICKS, /yaxis
    (*infoptr).yaxis->SetProperty, Range=contourkeywords.YRANGE, /yaxis
    (*infoptr).yaxis->SetProperty, Log=contourkeywords.YLOG, /yaxis

   ; Display the graphics.

    IAA_Contour_DrawTheContour, infoptr

    ENDIF


END ;-----------------------------------------------------------------------------


PRO IAA_Contour_Save_Settings, event

   ; This event handler responds to requests to save contour settings.

Widget_Control, event.top, Get_UValue=infoptr

TVLCT, r, g, b, /GET
r = congrid(r, 256, /Interp)
g = congrid(g, 256, /Interp)
b = congrid(b, 256, /Interp)

contourkeywords = (*infoptr).contourconfig->GetKeywords()

filename = DIALOG_PICKFILE(/WRITE, FILE = 'contoursettings.sav')

SAVE, FILENAME = filename, contourkeywords, r, g, b


END ;-----------------------------------------------------------------------------

PRO IAA_Contour_Save_Now, event

   ; This event handler responds to requests to save contour settings.

Widget_Control, event.top, Get_UValue=infoptr

   ; Save Settings for this functionality

IF (*infoptr).functionality NE '' THEN BEGIN

    TVLCT, r, g, b, /GET
    r = congrid(r, 256, /Interp)
    g = congrid(g, 256, /Interp)
    b = congrid(b, 256, /Interp)

    contourkeywords = (*infoptr).contourconfig->GetKeywords()

    filename = Filepath(Root_Dir=ProgramRootDir(), Subdirectory=['savfiles'], (*infoptr).functionality+'.sav')
    SAVE, FILENAME = filename, contourkeywords, r, g, b

    ok = DIALOG_MESSAGE('Settings have been saved.', /information)

ENDIF

END ;-----------------------------------------------------------------------------


PRO IAA_Contour_Motion, event

   ; This event handler responds to mouse motion events

Widget_Control, event.top, Get_UValue=infoptr

WSET, (*infoptr).wid

data = CONVERT_COORD(event.x, event.y, /DEVICE, /TO_DATA)

label = 'X: '+string(data[0])+'   Y: '+string(data[1])
widget_control, (*infoptr).labelID, set_value=label


END ;-----------------------------------------------------------------------------


PRO IAA_Contour_Resize, event

thisEvent = Tag_Names(event, /Structure_Name)

IF thisEvent EQ 'WIDGET_BASE' THEN BEGIN

      ; This event handler responds to TLB re-size events.

   Widget_Control, event.top, Get_UValue=infoptr

      ; Make the display window the current graphics window.

   WSet, (*infoptr).wid

      ; Resize the draw widget.

   Widget_Control, (*infoptr).drawID, Draw_XSize=(*infoptr).xminsize > event.x, $
      Draw_YSize=(*infoptr).yminsize > event.y

      ; Display the graphics.

   IAA_Contour_DrawTheContour, infoptr
   RETURN
ENDIF

IF thisEvent EQ 'WIDGET_KBRD_FOCUS' THEN BEGIN

   ; This event handler responds to TLB keyboard focus events.

   IF event.enter EQ 0 THEN RETURN

   Widget_Control, event.top, Get_UValue=infoptr

      ; Make the display window the current graphics window.

   WSet, (*infoptr).wid

   RETURN
ENDIF

END ;-----------------------------------------------------------------------------


PRO IAA_Contour_Configuration_Events, event

   ; This event handler responds to events comimg from
   ; the Contour Configuration widget via the buttonbase widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /cancel
   ok = Error_Message(/Traceback)
ENDIF
Widget_Control, event.top, Get_UValue=infoptr

   ; Display the graphics.

IAA_Contour_DrawTheContour, infoptr

END ;-----------------------------------------------------------------------------

PRO IAA_Contour_SetColorTable, event

      ; What kind of event is this?

   thisEvent = Tag_Names(event, /Structure_Name)

    ; Do the right thing. :-)

   CASE thisEvent OF

      'WIDGET_BUTTON': XColors, Group_Leader=event.top, $
         NotifyID=[event.id, event.top]

      'XCOLORS_LOAD': BEGIN

         Widget_Control, event.top, Get_UValue=infoptr


            WSet, (*infoptr).wid
            IAA_Contour_DrawTheContour, infoptr


        ENDCASE

   ENDCASE

END ;-----------------------------------------------------------------------------

PRO IAA_Contour, $
 zdata, $
 xdata, $
 ydata, $
 XTitle=xtitle, $
 YTitle=ytitle, $
 Title=title, $
 Nlevels=nlevels, $
 Min_value=min_value, $
 Max_value=max_value, $
 Color=color, $
 Background=background, $
 XLog=xlog, $
 YLog=ylog, $
 Cb_Log=cb_log, $
 C_LineStyle=c_linestyle, $
 Group_Leader=group_leader, $
 Functionality=functionality, $
 _Extra=extra

   ; Error handling.

Catch, theError
IF theError NE 0 THEN BEGIN
   Catch, /Cancel
   ok = Error_Message(/Traceback)
   RETURN
ENDIF

   ; Check color keywords. Differences for display depth

Device, Get_Visual_Depth=theDepth
IF theDepth GT 8 THEN BEGIN
      IF N_Elements(background) EQ 0 THEN background = 'IVORY'
      IF N_Elements(color) EQ 0 THEN color = 'SADDLE BROWN'
      device, decompose=0
ENDIF ELSE BEGIN
      IF N_Elements(background) EQ 0 THEN background = 'GRAY'
      IF N_Elements(color) EQ 0 THEN color = 'GREEN'
ENDELSE

   ; Build function(x,y) for contour.

IF N_Elements(zdata) EQ 0 THEN BEGIN

    zdata=DIST(100)
    xdata=findgen(100)
    ydata=findgen(100)

ENDIF ELSE BEGIN

    IF N_Elements(xdata) EQ 0 THEN Message, 'X, Y arrays must be specified.', /NoName

ENDELSE

   ; Check keywords.

IF N_Elements(title) EQ 0 THEN title=''
IF N_Elements(xtitle) EQ 0 THEN xtitle=''
IF N_Elements(ytitle) EQ 0 THEN ytitle=''
IF N_Elements(functionality) EQ 0 THEN functionality=''
IF N_Elements(c_linestyle) GT 0 THEN c_linestyle=1
IF N_Elements(nlevels) EQ 0 THEN nlevels=15
IF N_Elements(cb_log) EQ 0 THEN cb_log=0
IF Size(background, /TName) NE 'STRING' THEN Message, 'BACKGROUND must be specified as color NAME.', /NoName
IF Size(color, /TName) NE 'STRING' THEN Message, 'COLOR must be specified as color NAME.', /NoName

  ; Check for availability of GIF files.

thisVersion = Float(!Version.Release)
IF thisVersion LT 5.4 THEN haveGif = 1 ELSE haveGIF = 0

   ; Find the data axes ranges.

minx = Min(xdata, Max=maxx)
miny = Min(ydata, Max=maxy)
minz = Min(zdata, Max=maxz)

IF N_Elements(min_value) EQ 0 THEN min_value=minz
IF N_Elements(max_value) EQ 0 THEN max_value=maxz

   ; Define levels

IF cb_log EQ 1 THEN BEGIN

   logindex = WHERE(zdata GT 0)
   logdata = zdata[logindex]

   IF N_Elements(min_value) EQ 0 OR min_value EQ 0 THEN min_value = Min(logdata)
   nlevels = (CEIL(ALOG10(max_value))-FLOOR(ALOG10(min_value)))+1
   LevelsArr = FltArr(nlevels)

   step = 10.^CEIL(ALOG10(max_value))
   FOR i=0,nlevels-1 DO BEGIN
   LevelsArr[i]=step
   step = step/10
   ENDFOR

   LevelsArr = Reverse(LevelsArr)

ENDIF ELSE BEGIN

   LevelsArr = FltArr(nlevels)

   step = (max_value - min_value)/(nlevels*1.)
   FOR i=0,nlevels-1 DO BEGIN
   LevelsArr[i]=min_value+i*step
   ENDFOR

ENDELSE


   ; Create the widgets.

tlb = Widget_Base(Column=1, TLB_Size_Events=1, Base_Align_Center=1, KBRD_Focus_Events=1)
drawid = Widget_Draw(tlb, XSize=800, YSize=600, Event_Pro='IAA_Contour_Motion', /motion_events)
buttonbase = Widget_Base(tlb, Row=1)

   ; Quit button.

button = Widget_Button(buttonbase, Value='Quit', Event_Pro='IAA_Contour_Quit', /Dynamic_Resize)

    ; Restore button.

IF functionality NE '' THEN $
button = Widget_Button(buttonbase, Value='Restore Settings', Event_Pro='IAA_Contour_Restore', /Dynamic_Resize)

    ; Load button.

button = Widget_Button(buttonbase, Value='Load Settings', Event_Pro='IAA_Contour_Load', /Dynamic_Resize)

   ; Color Table Button.

button = Widget_Button(buttonbase, Value='Set Color Table', Event_Pro='IAA_Contour_SetColorTable')

   ; Print button.

printID = Widget_Button(buttonbase, Value='Print', Event_Pro='IAA_Contour_Print', /Menu, /Dynamic_Resize)
   dummy = Widget_Button(printID, Value='Landscape Orientation', UVALUE='LANDSCAPE')
   dummy = Widget_Button(printID, Value='Portrait Orientation', UVALUE='PORTRAIT')

   ; Save As button.

saveID = Widget_Button(buttonbase, Value='Save As', Event_Pro='IAA_Contour_SaveAs', /Menu, /Dynamic_Resize)
dummy = Widget_Button(saveID, Value='BMP File', UValue='BMP')
IF havegif THEN dummy = Widget_Button(saveID, Value='GIF File', UValue='GIF')
dummy = Widget_Button(saveID, Value='PICT File', UValue='PICT')
dummy = Widget_Button(saveID, Value='PNG File', UValue='PNG')
dummy = Widget_Button(saveID, Value='JPEG File', UValue='JPG')
dummy = Widget_Button(saveID, Value='TIFF File', UValue='TIF')
psID = Widget_Button(saveID, Value='PostScript File', UValue='PS', $
   Event_Pro='IAA_Contour_Postscript')

   ; Modify Contour button.

button = Widget_Button(buttonbase, Value='Modify Contour', $
  Event_Pro='IAA_Contour_Configure_Button', /Dynamic_Resize)

   ; Save Settings button.

button = Widget_Button(buttonbase, Value='Save Settings As..', $
  Event_Pro='IAA_Contour_Save_Settings', /Dynamic_Resize)

   ; Save Settings

IF functionality NE '' THEN $
button = Widget_Button(buttonbase, Value='Save Settings', Event_Pro='IAA_Contour_Save_Now', /Dynamic_Resize)

  ; About button.

button = Widget_Button(buttonbase, Value='About...', $
  Event_Pro='IAA_Contour_About', /Dynamic_Resize)

   ; Set up a widget to receive the Contour Configuration Events. The
   ; Buttonbase is not being used currently, so is a good choice.

Widget_Control, buttonBase, Event_Pro='IAA_Contour_Configuration_Events'

   ; Print Data
labelbase = Widget_Base(tlb, Row=1)
labelID = widget_label(labelbase, value='  ', /align_center, /dynamic_resize)

   ; Create the axis and contour configuration objects.

xaxis = Obj_New('IAA_Axis', Range=[minx, maxx],  /XAxis, _Extra=extra, Title=xtitle, $
   Log=Keyword_Set(xlog))
yaxis = Obj_New('IAA_Axis', Range=[miny, maxy], /YAxis, _Extra=extra, Title=ytitle, $
   Log=Keyword_Set(ylog))
contourconfig=Obj_New('IAA_ContourConfig', XAxis=xaxis, YAxis=yaxis, Barminval=min_value, Barmaxval=max_value, $
   _Extra=extra, Title=title, Color=color, Background=background)


   ; Realize the program and make the display window the current graphics window.

Widget_Control, tlb, /Realize, Update=0
Widget_Control, drawid, Get_Value=wid

   ; Get Keywords

contourkeywords = contourconfig->GetKeywords()
IF CB_LOG EQ 1 THEN contourconfig->SetProperty, CB_LOG=1
IF CB_LOG EQ 1 AND contourkeywords.barminval EQ 0 THEN contourconfig->SetProperty, barminval=1
contourconfig->SetProperty, nlevels=nlevels

    ; Calculate the contour colors.

step = !D.Table_Size / Float(contourkeywords.nlevels)
contourColors = Indgen(contourkeywords.nlevels) * step

   ; Draw the contour.

WSet, wid

IF contourkeywords.FILL EQ 0 THEN BEGIN

    Contour, zdata, xdata, ydata, C_COLORS=contourColors, Min_value=contourkeywords.barminval, $
    Max_value=contourkeywords.barmaxval, Levels=LevelsArr, _Extra=contourkeywords

ENDIF ELSE BEGIN

    IF contourkeywords.OVERPLOT GT 0 THEN BEGIN
       contourkeywords.OVERPLOT=0
       Contour, zdata, xdata, ydata, Min_value=contourkeywords.barminval, Max_value=contourkeywords.barmaxval, /closed, /cell_fill, $
       Levels=LevelsArr, _Extra=contourkeywords
       contourkeywords.FILL=0
       contourkeywords.OVERPLOT=1
       Contour, zdata, xdata, ydata, C_COLORS=contourkeywords.color,  Min_value=contourkeywords.barminval, $
       Levels=LevelsArr, Max_value=contourkeywords.barmaxval, _Extra=contourkeywords
       contourkeywords.FILL=1
    ENDIF ELSE BEGIN
       Contour, zdata, xdata, ydata, Min_value=contourkeywords.barminval, Max_value=contourkeywords.barmaxval, /closed, /cell_fill, $
       Levels=LevelsArr, _Extra=contourkeywords
    ENDELSE

ENDELSE


colorbarposition = contourkeywords.position
colorbarposition[0] = colorbarposition[2]+0.05
colorbarposition[2] = colorbarposition[0]+0.05

VCOLORBAR, BOTTOM=3,NCOLORS=!D.Table_Size-7, POSITION=colorbarposition, /VERTICAL, /RIGHT, CHARSIZE=contourkeywords.CHARSIZE, $
      CHARTHICK=contourkeywords.CHARTHICK, FONT=contourkeywords.FONT, TICKLEN = contourkeywords.TICKLEN*8, $
      DIVISIONS=contourkeywords.BARDIVISIONS, MINOR=contourkeywords.BARMINOR, FORMAT=contourkeywords.BARFORMAT, $
      TITLE=contourkeywords.BARTITLE, MINRANGE=contourkeywords.BARMINVAL, MAXRANGE=contourkeywords.BARMAXVAL, $
      CB_LOG=contourkeywords.CB_LOG, INVERTCOLORS=contourkeywords.INVERTCOLORS, COLOR=contourkeywords.color



Widget_Control, tlb, Update=1

; Inquire about buttonbase sizes. Needed for resizing events.

geom = Widget_Info(buttonbase, /Geometry)
yminsize = geom.scr_ysize + 300
xminsize = geom.scr_xsize

   ; Create an info structure and store it in the UValue of the TLB.

info = {xaxis:xaxis, yaxis:yaxis, wid:wid, zdata:zdata, xdata:xdata, ydata:ydata, contourconfig:contourconfig, $
   buttonbase:buttonbase, drawid:drawid, yminsize:yminsize, xminsize:xminsize, labelID:labelID, functionality:functionality}
infoPtr = Ptr_New(info, /No_Copy)
Widget_Control, tlb, set_uvalue=infoPtr

XManager, 'iaa_contour', tlb, /No_Block, Cleanup='IAA_Contour_Cleanup', $
   Event_Handler='IAA_Contour_Resize', Group_Leader=group_leader
END
