;+
; NAME:
;       IAA_CONTOURCONFIG__DEFINE
;
; PURPOSE:
;
;       This is a program for interactively adjusting and keeping track
;       of keywords appropriate for configuring the CONTOUR command.
;
; AUTHOR:
;
;       Jos Enrique Ruiz
;       Inspired upon David Fanning's MPI_PLOTCONFIG__DEFINE
;
; CATEGORY:
;
;       Graphics
;
; CALLING SEQUENCE:
;
;       contourConfigObj = Obj_New("IAA_CONTOURCONFIG")
;
; INPUT PARAMETERS:
;
;       None.
;
; INPUT KEYWORDS (Sent to the INIT method. The same keywords can be set with the SETPROPERTY method of the object.):
;
;      BACKGROUND - The name of the background color. By default on 24-bit systems: 'IVORY'. ON 8-bit systems 'GRAY'.
;
;      CHARSIZE - The character size of the contour. By default, 1.0.
;
;      CHARTHICK - The character thickness of the contour. By default, 1.0.
;
;      COLOR - The name of the contour color.
;         By default on 24-bit systems: 'SADDLE BROWN'. Uses 'GREEN' on 8-bit systems.
;
;
;      _EXTRA - Extra keywords to be passed to IAA_AXIS objects used internally.
;
;      FONT - The type of contour font: -1=Hershey, 0=Hardware, 1=True-Type. By default, !P.FONT.;
;
;      C_LINESTYLE - The contour linestyle. By default, 0. Possible values are:
;         0 - Solid Line
;         1 - Dotted
;         2 - Dashed
;         3 - Dash Dot
;         4 - Dash Dot Dot
;         5 - Long Dash
;
;      NOAXISINFO - Set this keyword to inhibit axis information on the GUI. By default, 0.
;
;      POSITION - The position of the contour in the contour window in normalized coordinates. By default, [0.20, 0.15, 0.95, 0.95].
;
;      SUBTITLE - The contour subtitle. By default, ''.
;
;      TITLE - The contour title. By default, ''.
;
;      THICK - The contour line thickness. By default, 1.0.
;
;      TICKLEN - The contour tick length. By default, 0.02.
;
;      XAXIS - An IAA_AXIS object for the X axis. One is created by default, if not provided.
;
;      YAXIS - An IAA_AXIS object for the Y axis. One is created by default, if not provided.
;
; METHOD PROCEDURES:
;
;      GUI - This procedure method displays a graphical user interface that allows the user
;            to change contour configuration parameters. The following keywords can be used:
;
;            ALL_EVENTS - Set this keyword to have an event sent any time something in the
;               GUI changes. The default is to send an event only when the user hits the ACCEPT button.
;               Note that the NOTIFYID keyword must be used to generate events.
;
;            BLOCK - Set this keyword if you want to block the command line. The default is to NOT block the command line.
;
;            GROUP_LEADER - The group leader for this GUI. If this keyword is used, the program will be
;               distroyed when the group leader is destroyed.
;
;            LABELDEFAULTSIZE - The default screen size for a label. All labels are offsets from this size. 55 by default.
;               The purpose of this keyword is to allow the user to modify the look of the GUI if different
;               fonts are used.
;
;            LABELFONT - The name of a font to use for program labels.
;
;            NOTIFYID - A two-element array containing the widget identifier and top-level base ID of a widget
;               designated to receive an event from this program. The event structure will be defined and sent
;               like this:
;
;                    Widget_Control, notifyid[0], Send_Event={ IAA_CONTOURCONFIG_EVENT, $
;                                                              ID: notifyid[0], $
;                                                              TOP:notifyid[1], $
;                                                              HANDLER: 0L, $
;                                                              OBJECT: self }
;
;               Most event handlers will be written so that they will get the contour keywords
;               from the contour configuration object and draw the contour. A sample event handler might
;               look like this:
;
;                    PRO IAA_Contour_Configuration_Events, event
;                    Widget_Control, event.top, Get_UValue=info, /No_Copy
;                    WSet, info.wid
;                    contourkeywords = event.object->GetKeywords()
;                    Contour, info.zdata, info.xdata, info.ydata, _Extra=contourkeywords
;                    Widget_Control, event.top, Set_UValue=info, /No_Copy
;                    END
;
;            XLONGFORM - By default, the X axis information is displayed in "short" form, with only the
;               most relevant information readily available. Other axis information is accessed via buttons.
;               Set this keyword to display the X axis information in a "long" form, in which all the axis
;               information is immediately visible.
;
;            YLONGFORM - By default, the Y axis information is displayed in "short" form, with only the
;               most relevant information readily available. Other axis information is accessed via buttons.
;               Set this keyword to display the Y axis information in a "long" form, in which all the axis
;               information is immediately visible.
;
;      SETPROPERTY - This procedure can be used to set the properties of the contour
;               configuration object without using the graphical user interface. The
;               keywords are identical to those used in the INIT method, above.
;
; METHOD FUNCTIONS:
;
;      GETKEYWORDS - This function method contains no arguments or keywords. It returns a
;            structure, with fields equivalent to CONTOUR keywords. The idea is that these
;            keywords can be passed directly to the CONTOUR command using the keyword inheritance
;            mechanism via the _EXTRA keyword to the contour command. A possible sequence of commands
;            might look like this:
;
;               IDL> contourConfigObj = Obj_New("IAA_CONTOURCONFIG")   ; Create the contour configuration object.
;               IDL> contourConfigObj->GUI, /Block                  ; Allow the user to configure the contour parameters.
;               IDL> contourKeywords = contourConfigObj->GetKeywords() ; Get the contour keywords.
;               IDL> Contour z, x, y, _Extra=contourKeywords           ; Draw the contour in the way the user specified.
;               IDL> Obj_Destroy, contourConfigObj                  ; Destroy the object when finished with it.
;
; PROGRAM NOTES:
;
;      Color Names: Color names are those used with FSC_Color and PickColorName. See the
;         documentation for those programs for instuctions on loading your own colors.
;         To see the default colors and names, type this:
;
;                IDL> color = PickColorName('yellow')
;
;

PRO IAA_ContourConfig_CenterTLB, tlb

; Center the top-level base on the display.

screenSize = Get_Screen_Size()
IF screenSize[0] GT 2000 THEN screenSize[0] = screenSize[0]/2 ; Dual monitors.
xCenter = screenSize(0) / 2
yCenter = screenSize(1) / 2

geom = Widget_Info(tlb, /Geometry)
xHalfSize = geom.Scr_XSize / 2
yHalfSize = geom.Scr_YSize / 2

Widget_Control, tlb, XOffset = xCenter-xHalfSize, $
   YOffset = yCenter-yHalfSize

END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig_Error_Message, theMessage, Traceback=traceback, NoName=noName, _Extra=extra

On_Error, 2

   ; Check for presence and type of message.

IF N_Elements(theMessage) EQ 0 THEN theMessage = !Error_State.Msg
s = Size(theMessage)
messageType = s[s[0]+1]
IF messageType NE 7 THEN BEGIN
   Message, "The message parameter must be a string.", _Extra=extra
ENDIF

   ; Get the call stack and the calling routine's name.

Help, Calls=callStack
callingRoutine = (StrSplit(StrCompress(callStack[1])," ", /Extract))[0]


   ; Are widgets supported? Doesn't matter in IDL 5.3 and higher.

widgetsSupported = ((!D.Flags AND 65536L) NE 0) OR Float(!Version.Release) GE 5.3
IF widgetsSupported THEN BEGIN
   IF Keyword_Set(noName) THEN answer = Dialog_Message(theMessage, _Extra=extra) ELSE BEGIN
      IF StrUpCase(callingRoutine) EQ "$MAIN$" THEN answer = Dialog_Message(theMessage, _Extra=extra) ELSE $
         answer = Dialog_Message(StrUpCase(callingRoutine) + ": " + theMessage, _Extra=extra)
   ENDELSE
ENDIF ELSE BEGIN
      Message, theMessage, /Continue, /NoPrint, /NoName, /NoPrefix, _Extra=extra
      Print, '%' + callingRoutine + ': ' + theMessage
      answer = 'OK'
ENDELSE

   ; Provide traceback information if requested.

IF Keyword_Set(traceback) THEN BEGIN
   Help, /Last_Message, Output=traceback
   Print,''
   Print, 'Traceback Report from ' + StrUpCase(callingRoutine) + ':'
   Print, ''
   FOR j=0,N_Elements(traceback)-1 DO Print, "     " + traceback[j]
ENDIF

RETURN, answer
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig::AxisObjectEvents, event

; This method responds to events from the axis objects, if they are
; associated with the contour object.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig::SetContourKeywords, contourkeywords

; This method sets the contour properties from a structure of keywords.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN
ENDIF

tagnames = Tag_Names(contourkeywords)
contourkeywords.color = contourkeywords.name_color
contourkeywords.background = contourkeywords.name_background



END ;-----------------------------------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig::Destroy

; This method destroys the top-level base of the compound widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN
ENDIF

IF Widget_Info(self.tlb, /Valid_ID) THEN Widget_Control, self.tlb, /Destroy
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::GetTLB

; This method returns the top-level base of the compound widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

RETURN, self.tlb
END ;-----------------------------------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig::SendEvent

; This method sends an event to the widget to be notified, if one exists.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN
ENDIF

IF Total(self.notifyID) GT 0 THEN BEGIN

   thisEvent = { IAA_CONTOURCONFIG_EVENT, $
                 ID: self.notifyID[0], $
                 TOP:self.notifyID[1], $
                 HANDLER: 0L, $
                 OBJECT: self }

   Widget_Control, self.notifyID[0], Send_Event=thisEvent
ENDIF

END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::GetKeywords

; This method constructs a structure containing fields with values
; set to the current state of the object.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self->UpdateObject
struct = Create_Struct( $
   'nlevels', self.nlevels, $
   'background', FSC_Color(self.background, !P.Background), $
   'charsize', self.charsize, $
   'charthick', self.charthick, $
   'color', FSC_Color(self.color, !D.Table_Size-1), $
   'font', self.font, $
   'position', self.position, $
   'subtitle', self.subtitle, $
   'title', self.title, $
   'bartitle', self.bartitle, $
   'c_linestyle', self.c_linestyle, $
   'thick', self.thick, $
   'ticklen', self.ticklen, $
   'fill', self.fill, $
   'follow', self.follow, $
   'overplot', self.overplot, $
   'noerase', self.noerase, $
   'name_color', self.color, $
   'name_background', self.background, $
   'bardivisions', self.bardivisions, $
   'barminor', self.barminor, $
   'barformat', self.barformat, $
   'barminval', self.barminval, $
   'barmaxval', self.barmaxval, $
   'invertcolors', self.invertcolors, $
   'cb_log', self.cb_log, $
   'irregular', self.irregular, $
   'isotropic', self.isotropic, $
   'downhill', self.downhill $
    )

   ; Done unless axis information is required.

IF self.noaxisinfo THEN RETURN, struct

IF Obj_Valid(self.xaxis) THEN BEGIN
   xstruct = self.xaxis->GetKeywords()
   struct = Create_Struct(struct, xstruct)
ENDIF

IF Obj_Valid(self.yaxis) THEN BEGIN
   ystruct = self.yaxis->GetKeywords()
   struct = Create_Struct(struct, ystruct)
ENDIF

RETURN, struct
END ;-----------------------------------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig::UpdateObject

; This method simply gathers all the information in the graphical
; user interface (that doesn't update itself) and updates the self
; object. If fields are undefined, they are set to their default values.

IF Obj_Valid(self.titleObj) NE 0 THEN BEGIN
   IF N_Elements(self.titleObj->Get_Value()) EQ 0 THEN BEGIN
      value = ''
      self.titleObj->Set_Value, value
   ENDIF ELSE value = self.titleObj->Get_Value()
self.title = value
ENDIF

IF Obj_Valid(self.subtitleObj) NE 0 THEN BEGIN
   IF N_Elements(self.subtitleObj->Get_Value()) EQ 0 THEN BEGIN
      value = ''
      self.subtitleObj->Set_Value, value
   ENDIF ELSE value = self.subtitleObj->Get_Value()
self.subtitle = value
ENDIF

IF Obj_Valid(self.thickObj) NE 0 THEN BEGIN
   IF Finite(self.thickObj->Get_Value()) EQ 0 THEN BEGIN
      value = 1.0
      self.thickObj->Set_Value, value
   ENDIF ELSE value = self.thickObj->Get_Value()
self.thick = value
ENDIF

IF Obj_Valid(self.charthickObj) NE 0 THEN BEGIN
   IF Finite(self.charthickObj->Get_Value()) EQ 0 THEN BEGIN
      value = 1.0
      self.charthickObj->Set_Value, value
   ENDIF ELSE value = self.charthickObj->Get_Value()
self.charthick = value
ENDIF

IF Obj_Valid(self.ticklenObj) NE 0 THEN BEGIN
   IF Finite(self.ticklenObj->Get_Value()) EQ 0 THEN BEGIN
      value = 0.020
      self.ticklenObj->Set_Value, value
   ENDIF ELSE value = self.ticklenObj->Get_Value()
self.ticklen = value
ENDIF

IF Obj_Valid(self.charsizeObj) NE 0 THEN BEGIN
   IF Finite(self.charsizeObj->Get_Value()) EQ 0 THEN BEGIN
      value = 1.0
      self.charsizeObj->Set_Value, value
   ENDIF ELSE value = self.charsizeObj->Get_Value()
self.charsize = value
ENDIF

IF Obj_Valid(self.nlevelsObj) NE 0 THEN BEGIN
   IF Finite(self.nlevelsObj->Get_Value()) EQ 0 THEN BEGIN
      value = 15
      self.nlevelsObj->Set_Value, value
   ENDIF ELSE value = self.nlevelsObj->Get_Value()
self.nlevels = value
ENDIF

IF Obj_Valid(self.bartitleObj) NE 0 THEN BEGIN
   IF N_Elements(self.bartitleObj->Get_Value()) EQ 0 THEN BEGIN
      value = ''
      self.bartitleObj->Set_Value, value
   ENDIF ELSE value = self.bartitleObj->Get_Value()
self.bartitle = value
ENDIF

IF Obj_Valid(self.bardivisionsObj) NE 0 THEN BEGIN
   IF Finite(self.bardivisionsObj->Get_Value()) EQ 0 THEN BEGIN
      value = 6
      self.bardivisionsObj->Set_Value, value
   ENDIF ELSE value = self.bardivisionsObj->Get_Value()
self.bardivisions = value
ENDIF

IF Obj_Valid(self.barminorObj) NE 0 THEN BEGIN
   IF Finite(self.barminorObj->Get_Value()) EQ 0 THEN BEGIN
      value = 2
      self.barminorObj->Set_Value, value
   ENDIF ELSE value = self.barminorObj->Get_Value()
self.barminor = value
ENDIF

IF Obj_Valid(self.barminvalObj) NE 0 THEN BEGIN
   IF Finite(self.barminvalObj->Get_Value()) EQ 0 THEN BEGIN
      value = 2
      self.barminvalObj->Set_Value, value
   ENDIF ELSE value = self.barminvalObj->Get_Value()
self.barminval = value
ENDIF

IF Obj_Valid(self.barmaxvalObj) NE 0 THEN BEGIN
   IF Finite(self.barmaxvalObj->Get_Value()) EQ 0 THEN BEGIN
      value = 2
      self.barmaxvalObj->Set_Value, value
   ENDIF ELSE value = self.barmaxvalObj->Get_Value()
self.barmaxval = value
ENDIF

IF Obj_Valid(self.barformatObj) NE 0 THEN BEGIN

   value = self.barformatObj->Get_Value()
   stringLen = StrLen(value)

   IF stringLen EQ 0 THEN BEGIN
      value = ''
      self.barformatObj->Set_Value, value

   ENDIF ELSE BEGIN

    IF StrMid(value, 0, 1) NE "(" THEN BEGIN
       value = "(" + value
       stringLen = StrLen(value)
    ENDIF
    IF StrMid(value, stringLen-1, 1) NE ")" THEN BEGIN
       value = value + ")"
    ENDIF
    self.barformatObj->Set_Value, value

   ENDELSE

self.barformat = value
ENDIF

END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::CheckEvent, event

; This event handler method checks to see if an event
; should be passed on.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

   ; Make sure all values are current by updating.

self->UpdateObject
IF self.all_events THEN self->SendEvent

RETURN, 0
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::SetContourPosition, event

; This event handler method sets the contour position.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

Widget_Control, event.top, TLB_Get_Offset=offsets, TLB_Get_Size=tlbsizes
self.position = AdjustPosition(self.position, Group_Leader=event.top, Cancel=cancelled, $
   XOffset=offsets[0] + 0.7*tlbsizes[0], YOffset=offsets[1] + 0.4*tlbsizes[1])

IF cancelled THEN RETURN, 0 ELSE RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig::SetTitle, event

; This event handler method sets the contour title.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.title = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::SetSubTitle, event

; This event handler method sets the contour subtitle.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.subtitle = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::SetFont, event

; This event handler method sets the contour font style.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.font = event.index - 1

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetContours, event

; This event handler method sets the contour font style.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF


CASE event.index OF
       0: BEGIN
            self.fill = 0
            self.follow = 0
            self.overplot = 0
          END
       1: BEGIN
            self.fill = 0
            self.follow = 1
            self.overplot = 0
          END
       2: BEGIN
            self.fill = 1
            self.follow = 0
            self.overplot = 0
          END
       3: BEGIN
            self.fill = 1
            self.follow = 0
            self.overplot = 1
          END
       4: BEGIN
            self.fill = 1
            self.follow = 1
            self.overplot = 1
          END
ENDCASE

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetLinestyle, event

; This event handler method sets the contour line style.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.c_linestyle = event.index

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetBackgroundColor, event

; This event handler method sets the contour background color.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.background = event.color

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig::SetColor, event

; This event handler method sets the contour color.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.color = event.color

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig::SetLineThick, event

; This event handler method sets the contour line thickness.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.thick = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::SetCharThick, event

; This event handler method sets the contour character thickness.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.charthick = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig::SetCharSize, event

; This event handler method sets the contour symbol size.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.charsize = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetTickLen, event

; This event handler method sets the contour tick length.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.ticklen = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetNLevels, event

; This event handler method sets the contour number of levels.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.nlevels = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetBarTitle, event

; This event handler method sets the contour title.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.bartitle = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig::SetBarDivisions, event

; This event handler method sets the number of bar divisions.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.bardivisions = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig::SetBarMinor, event

; This event handler method sets the number of bar minor ticks.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.barminor = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetBarMinVal, event

; This event handler method sets the min. value colorbar.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.barminval = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetBarMaxVal, event

; This event handler method sets the max. value colorbar.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.barmaxval = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetInvertColors, event

; This event handler method sets invertcolors colorbar flag.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

IF event.select THEN self.invertcolors = self.invertcolors + 1 ELSE self.invertcolors = self.invertcolors - 1

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetCb_Log, event

; This event handler method sets log scale colorbar.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

IF event.select THEN self.cb_log = self.cb_log + 1 ELSE self.cb_log = self.cb_log - 1

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_ContourConfig::SetBarFormat, event

; This event handler method sets the bar tick format.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = Error_Message(/Traceback)
   RETURN, 0
ENDIF


theFormat = *event.value
IF *event.value EQ '' THEN RETURN, self->CheckEvent(event)

stringLen = StrLen(theFormat)
change = 0
IF StrMid(theFormat, 0, 1) NE "(" THEN BEGIN
   theFormat = "(" + theFormat
   stringLen = StrLen(theFormat)
   change = 1
ENDIF
IF StrMid(theFormat, stringLen-1, 1) NE ")" THEN BEGIN
   theFormat = theFormat + ")"
   change = 1
ENDIF
IF change THEN event.object->Set_Value, theFormat

self.barformat = theFormat

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_Axis::SetBarFormat, event

; This event handler method sets the barcolor tick format.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = Error_Message(/Traceback)
   RETURN, 0
ENDIF


theFormat = *event.value
IF *event.value EQ '' THEN RETURN, self->CheckEvent(event)

stringLen = StrLen(theFormat)
change = 0
IF StrMid(theFormat, 0, 1) NE "(" THEN BEGIN
   theFormat = "(" + theFormat
   stringLen = StrLen(theFormat)
   change = 1
ENDIF
IF StrMid(theFormat, stringLen-1, 1) NE ")" THEN BEGIN
   theFormat = theFormat + ")"
   change = 1
ENDIF
IF change THEN event.object->Set_Value, theFormat

self.barformat = theFormat

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_ContourConfig::DismissDialog, event

; This event handler method destroys the widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

Widget_Control, self.tlb, /Destroy

RETURN, 0
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::AcceptDialog, event

; This event handler method updates the dialog and destroys the widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self->UpdateObject
IF Obj_Valid(self.xaxis) THEN self.xaxis->UpdateObject
IF Obj_Valid(self.yaxis) THEN self.yaxis->UpdateObject

self->SendEvent
self->Destroy

RETURN, 0
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::ApplyDialog, event

; This event handler method updates the dialog and destroys the widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self->UpdateObject
IF Obj_Valid(self.xaxis) THEN self.xaxis->UpdateObject
IF Obj_Valid(self.yaxis) THEN self.yaxis->UpdateObject

self->SendEvent

RETURN, 0
END ;-----------------------------------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig::GUI, $

; This method displays a graphical user interface for changing the values in the object.

   All_events=all_events, $             ; Set this keyword to get all of the events generated.
   Block=block, $                       ; Set this keyword if you want to block the command line.
   Group_Leader=group_leader, $         ; The group leader for this GUI.
   LabelDefaultSize=labeldefaultsize, $ ; The default screen size for a label. All labels are offsets from this size. 55 by default.
   LabelFont=labelfont, $               ; The name of a font to use for program labels.
   NotifyID=notifyID, $                 ; The widget ID and top-level base to notify upon an event.
   XLongForm=xlongform, $               ; Set this keyword to display the X axis information in a long form.
   YLongForm=ylongform                  ; Set this keyword to display the Y axis information in a long form.


   ; Error handling.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN
ENDIF

   ; Only one version of this program at a time.

IF XRegistered('iaa_contourconfig') GT 0 THEN RETURN

   ; Check keyword arguments.

self.all_events = Keyword_Set(all_events)
IF N_Elements(notifyID) NE 0 THEN self.notifyID = notifyID

   ; Set up the default label size.

IF N_Elements(labeldefaultsize) EQ 0 THEN labeldefaultsize = 70

   ; Create the widget interface.

IF N_Elements(group_leader) EQ 0 THEN BEGIN
   tlb = Widget_Base(Column=1, Title='Contour Options...', $ ;Base_Align_Center=1
      Pro_Set_Value="IAA_CONTOUR_SET_VALUE", Func_Get_Value="IAA_CONTOUR_GET_VALUE", Map=0, /tlb_frame_attr)
ENDIF ELSE BEGIN
   tlb = Widget_Base(Column=1, Title='Contour Options...', $ ;Base_Align_Center=1, $
      Pro_Set_Value="IAA_CONTOUR_SET_VALUE", Func_Get_Value="IAA_CONTOUR_GET_VALUE", $
      Group_Leader=group_leader, Map=0, /tlb_frame_attr)
ENDELSE

mainbase = Widget_Base(tlb, Column=1, Frame=1, UValue=self, Base_Align_Center=1)


   label = Widget_Label(mainbase, Value='Contour Options', Font=self.labelfont)

col1 = Widget_Base(mainbase, Column=1)
row1 = Widget_Base(col1, Row=1)

   titleID = FSC_Field(row1, LabelSize=labeldefaultsize, Title='Contour Title: ', $
      Label_Left=1, Value=self.title, XSize=40, Object=titleObj, $
      UValue={method:'SETTITLE', object:self}, $
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

row2 = Widget_Base(col1, Row=1)

   subtitleID = FSC_Field(row2, LabelSize=labeldefaultsize, Title='Subtitle: ', $
      Label_Left=1, Value=self.subtitle, XSize=40, Object=subtitleObj, $
      UValue={method:'SETSUBTITLE', object:self}, $
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

   droprow = Widget_Base(mainbase, Row=1, Base_Align_Center=1)

   fonttype = ['Hershey', 'Hardware', 'True-Type']
   CASE self.font OF
      -1: fontIndex = 0
       0: fontIndex = 1
       1: fontIndex = 2
   ENDCASE
   fontID = FSC_Droplist(droprow, Value=fonttype, Index=fontIndex, $
      UValue={method:'SetFont', object:self}, Title='Font Type:', Space=[1,1])

   styles = ['Solid', 'Dotted', 'Dashed', 'Dash Dot', 'Dash Dot Dot', 'Long Dash']
   linestyleID = FSC_Droplist(droprow, Value=styles, Index=self.c_linestyle, $
      UValue={method:'SetLineStyle', object:self}, Title='Line Style:', Space=[1,1])

   contours = ['Lines', 'Lines and labels', 'Filled', 'Filled and lines', $
              'Filled, lines and labels']

   IF (self.fill) THEN BEGIN

        IF (self.follow AND self.overplot) THEN BEGIN
          contourIndex=4
        ENDIF ELSE BEGIN

        IF (self.overplot) THEN BEGIN
              contourIndex=3
            ENDIF ELSE BEGIN
              contourIndex=2
            ENDELSE
        ENDELSE

   ENDIF ELSE BEGIN
        IF (self.follow) THEN BEGIN
          contourIndex=1
        ENDIF ELSE BEGIN
          contourIndex=0
        ENDELSE
   ENDELSE

   contoursID = FSC_Droplist(droprow, Value=contours, Index=contourIndex, $
      UValue={method:'SetContours', object:self}, Title='Plot Style:', Space=[1,1])


   row4 = Widget_Base(mainbase, Row=1)
   column1 = Widget_Base(row4, Column=1)
   column2 = Widget_Base(row4, Column=1)
   column3 = Widget_Base(row4, Column=1)

   thickID = FSC_Field(column1, Value=Float(self.thick), Title='Line Thickness: ', $
      Label_Left=1, Object=thickObj, XSize=6, Decimal=3, $
      UValue={method:'SETLINETHICK', object:self}, LabelSize=labeldefaultsize*2, $   ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

   charthickID = FSC_Field(column1, Value=Float(self.charthick), Title='Character Thickness: ', $
      Label_Left=1, Object=charthickObj, XSize=6, Decimal=3, $
      UValue={method:'SETCHARTHICK', object:self}, LabelSize=labeldefaultsize*2, $    ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

   nlevelsID = FSC_Field(column2, Value=self.nlevels, Title='Levels: ', $
      Label_Left=1, Object=nlevelsObj, XSize=6, Decimal=0, $
      UValue={method:'SETNLEVELS', object:self}, LabelSize=labeldefaultsize+25, $    ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

   charsizeID = FSC_Field(column2, Value=Float(self.charsize), Title='Character Size: ', $
      Label_Left=1, Object=charsizeObj, XSize=6, Decimal=3, $
      UValue={method:'SETCHARSIZE', object:self}, LabelSize=labeldefaultsize+25, $      ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

   ticklenID = FSC_Field(column3, Value=Float(self.ticklen), Title='Tick Length: ', $
      Label_Left=1, Object=ticklenObj, XSize=6, Decimal=3, $
      UValue={method:'SETTICKLEN', object:self}, $
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

   button = Widget_Button(column3, Value='Contour Position', $
      Event_Pro="IAA_ContourConfig_Widget_Events", UValue={method:'SETCONTOURPOSITION', object:self})


   row5 = Widget_Base(mainbase, Row=1)
      colorID = CW_Drawcolor(row5, Color=self.color, LabelText='Color: ', Title='Color:', $
         UValue={method:'SETCOLOR', object:self}, Label_Left=1, Object=colorObj)
      bcolorID = CW_Drawcolor(row5, Color=self.background, LabelText='   Background Color: ', Title='Background Color', $
         UValue={method:'SETBACKGROUNDCOLOR', object:self}, Label_Left=1, Object=bcolorObj)


    barbox = Widget_Base(mainbase, Column=1, /tlb_frame_attr)
    label = Widget_Label(barbox, Value='Barcolor Properties', Font=self.labelfont)

    bartitle = Widget_Base(barbox, Row=1)

    titlebarID = FSC_Field(bartitle, LabelSize=labeldefaultsize, Title='Colorbar Title: ', $
      Label_Left=1, Value=self.bartitle, XSize=40, Object=bartitleObj, $
      UValue={method:'SETBARTITLE', object:self}, $
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

    barinvert = Widget_Base(bartitle, Row=1, /Nonexclusive)
    invertcolorsID = Widget_Button(barinvert, Value='Invert', UValue={method:'SETINVERTCOLORS', object:self}, $
      Event_Pro='IAA_ContourConfig_Widget_Events')
    IF self.invertcolors THEN Widget_Control, invertcolorsID, /Set_Button

    barpropperties = Widget_Base(barbox, Row=1)
    bardivisionsID = FSC_Field(barpropperties, Value=self.bardivisions, Title='Divisions: ', $
      Label_Left=1, Object=bardivisionsObj, XSize=6, Decimal=0, $
      UValue={method:'SETBARDIVISIONS', object:self}, LabelSize=labeldefaultsize, $    ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")
    barminorID = FSC_Field(barpropperties, Value=self.barminor, Title='Minor Ticks: ', $
      Label_Left=1, Object=barminorObj, XSize=6, Decimal=0, $
      UValue={method:'SETBARMINOR', object:self}, LabelSize=labeldefaultsize, $    ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")
    barformatID = FSC_Field(barpropperties, Value=self.barformat, Title='Format: ', $
      Label_Left=1, Object=barformatObj, XSize=6,$
      UValue={method:'SETBARFORMAT', object:self}, LabelSize=labeldefaultsize5, $    ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

    barpropperties2 = Widget_Base(barbox, Row=1)
    barminvalID = FSC_Field(barpropperties2, Value=self.barminval, Title='Min. value: ', $
      Label_Left=1, Object=barminvalObj, XSize=12, Decimal=10, $
      UValue={method:'SETBARMINVAL', object:self}, LabelSize=labeldefaultsize, $    ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")
    barmaxvalID = FSC_Field(barpropperties2, Value=self.barmaxval, Title='Max. value: ', $
      Label_Left=1, Object=barmaxvalObj, XSize=12, Decimal=10, $
      UValue={method:'SETBARMAXVAL', object:self}, LabelSize=labeldefaultsize, $    ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_ContourConfig_Widget_Events")

    barlog = Widget_Base(barpropperties2, Row=1, /Nonexclusive)
    cb_logID = Widget_Button(barlog, Value='Logarithmic', UValue={method:'SETCB_LOG', object:self}, $
      Event_Pro='IAA_ContourConfig_Widget_Events')
    IF self.cb_log THEN Widget_Control, cb_logID, /Set_Button

    ; The X and Y axis information if required and available.

   IF self.noaxisinfo EQ 0 THEN BEGIN
      IF self.all_events THEN event_pro = 'IAA_ContourConfig_Widget_Events' ELSE event_pro="
      IF Obj_Valid(self.xaxis) THEN BEGIN
         xaxisID = self.xaxis->GUI(tlb, Short=1-Keyword_Set(xlongform), Event_Pro=event_pro)
         Widget_Control, xaxisID, Set_UValue={method:'AXISOBJECTEVENTS', object:self}
      ENDIF
      IF Obj_Valid(self.yaxis) THEN BEGIN
         yaxisID = self.yaxis->GUI(tlb, Short=1-Keyword_Set(ylongform), Event_Pro=event_pro)
         Widget_Control, yaxisID, Set_UValue={method:'AXISOBJECTEVENTS', object:self}
      ENDIF
   ENDIF



  buttonbase = Widget_Base(tlb, Row=1, Align_Center=1)
  button = Widget_Button(buttonbase, Value='Close', UValue={Object:self, Method:'DISMISSDIALOG'})
  IF Keyword_Set(block) THEN BEGIN
     button = Widget_Button(buttonbase, Value='Accept', UValue={Object:self, Method:'ACCEPTDIALOG'})
  ENDIF ELSE BEGIN
     button = Widget_Button(buttonbase, Value='Apply', UValue={Object:self, Method:'APPLYDIALOG'})
  ENDELSE

IAA_ContourConfig_CenterTLB, tlb

Widget_Control, tlb, /Realize


   ; Fill out object fields.

self.tlb = tlb
self.titleObj = titleObj
self.subtitleObj = subtitleObj
self.thickObj = thickObj
self.ticklenObj = ticklenObj
self.charthickObj = charthickObj
self.charsizeObj = charsizeObj
self.nlevelsObj = nlevelsObj
self.colorObj = colorObj
self.bcolorObj = bcolorObj

self.lineStyleID = linestyleID
self.bartitleObj = bartitleObj
self.bardivisionsObj = bardivisionsObj
self.barminorObj = barminorObj
self.barformatObj = barformatObj
self.barminvalObj = barminvalObj
self.barmaxvalObj = barmaxvalObj

Widget_Control, tlb, Map=1
XManager, 'iaa_contourconfig', tlb, Event_Handler='IAA_ContourConfig_Widget_Events', No_Block = 1 - Keyword_Set(block)

END ;-----------------------------------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig::SetProperty, $

; This method allows you to set the property of the object
; without going through the graphical user interface.

   Nlevels=nlevels, $           ; The number of levels
   Background=background, $     ; The name of the background color
   Charsize=charsize, $         ; The character size of the contour.
   Charthick=charthick, $       ; The character thickness of the contour.
   Color=color, $               ; The name of the contour color.
   Font=font, $                 ; The type of contour font: -1=Hershey, 0=Hardware, 1=True-Type.
   Position=position, $         ; The position of the contour in the contour window.
   Subtitle=subtitle, $         ; The contour subtitle.
   Title=title, $               ; The contour title.
   C_linestyle=c_linestyle, $   ; The contour linestyle.
   Thick=thick, $               ; The contour line thickness.
   Ticklen=ticklen, $           ; The contour tick length.
   Fill=fill, $                 ; The contour fill value.
   Follow=follow, $             ; The contour follow value.
   Overplot=overplot, $         ; The contour follow value.
   Noerase=noerase, $           ; The contour noerase value.
   Bartitle=bartitle, $         ; The barcolor title.
   Bardivisions=bardivisions, $ ; The barcolor divisions.
   Barminor=barminor, $         ; The barcolor minor ticks.
   Barformat=barformat, $       ; The barcolor format.
   Barminval=barminval, $       ; The barcolor min. value.
   Barmaxval=barmaxval, $       ; The barcolor max.value.
   Invertcolors=invertcolors, $ ; The invertcolors barcolor flag.
   Cb_Log=cb_log, $             ; The barcolor log scale oprion.
   Irregular=irregular, $       ; Irregular grid contour.
   Isotropic=isotropic, $       ; Isotropic grid contour.
   Downhill=downhill, $         ; Downhill grid contour.
   NoAxisInfo=noaxisinfo, $     ; Set this keyword to inhibit axis information on the GUI.
   XAxis=xaxis, $               ; An IAA_AXIS object for the X axis.
   YAxis=yaxis                  ; An IAA_AXIS object for the Y axis.


IF N_Elements(nlevels) NE 0 THEN self.nlevels = nlevels
IF N_Elements(background) NE 0 THEN self.background = background
IF N_Elements(charsize) NE 0 THEN self.charsize = charsize
IF N_Elements(charthick) NE 0 THEN self.charthick = charthick
IF N_Elements(color) NE 0 THEN self.color = color
IF N_Elements(font) NE 0 THEN self.font = font
IF N_Elements(position) NE 0 THEN self.position = position
IF N_Elements(subtitle) NE 0 THEN self.subtitle = subtitle
IF N_Elements(title) NE 0 THEN self.title = title
IF N_Elements(c_linestyle) NE 0 THEN self.c_linestyle = c_linestyle
IF N_Elements(thick) NE 0 THEN self.thick = thick
IF N_Elements(ticklen) NE 0 THEN self.ticklen = ticklen
IF N_Elements(fill) NE 0 THEN self.fill = fill
IF N_Elements(follow) NE 0 THEN self.follow = follow
IF N_Elements(overplot) NE 0 THEN self.overplot = overplot
IF N_Elements(noerase) NE 0 THEN self.noerase = noerase

IF N_Elements(bartitle) NE 0 THEN self.bartitle = bartitle
IF N_Elements(bardivisions) NE 0 THEN self.bardivisions = bardivisions
IF N_Elements(barminor) NE 0 THEN self.barminor = barminor
IF N_Elements(barformat) NE 0 THEN self.barformat = barformat
IF N_Elements(barminval) NE 0 THEN self.barminval = barminval
IF N_Elements(barmaxval) NE 0 THEN self.barmaxval = barmaxval
IF N_Elements(invertcolors) NE 0 THEN self.invertcolors = invertcolors
IF N_Elements(cb_log) NE 0 THEN self.cb_log = cb_log

IF N_Elements(noaxisinfo) NE 0 THEN self.noaxisinfo = Keyword_Set(noaxisinfo)
IF N_Elements(irregular) NE 0 THEN self.irregular = irregular
IF N_Elements(isotropic) NE 0 THEN self.isotropic = isotropic
IF N_Elements(downhill) NE 0 THEN self.downhill = downhill


IF N_Elements(xaxis) NE 0 THEN BEGIN
   IF self.createaxis THEN Obj_Destroy, self.xaxis
   self.xaxis = xaxis
ENDIF
IF N_Elements(yaxis) NE 0 THEN BEGIN
   IF self.createaxis THEN Obj_Destroy, self.yaxis
   self.yaxis = yaxis
ENDIF

self->UpdateObject
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig::Init, $

; This is the initialization method for the IAA_CONTOURCONFIG object.

   Nlevels=nlevels, $           ; The number of levels
   Background=background, $     ; The name of the background color
   Charsize=charsize, $         ; The character size of the contour.
   Charthick=charthick, $       ; The character thickness of the contour.
   Color=color, $               ; The name of the contour color.
   Font=font, $                 ; The type of contour font: -1=Hershey, 0=Hardware, 1=True-Type.
   Position=position, $         ; The position of the contour in the contour window.
   Subtitle=subtitle, $         ; The contour subtitle.
   Title=title, $               ; The contour title.
   C_linestyle=c_linestyle, $   ; The contour linestyle.
   Thick=thick, $               ; The contour line thickness.
   Ticklen=ticklen, $           ; The contour tick length.
   Fill=fill, $                 ; The contour fill value.
   Follow=follow, $             ; The contour follow value.
   Overplot=overplot, $         ; The contour noerase value
   Noerase=noerase, $           ; The contour noerase value.
   Bartitle=bartitle, $         ; The barcolor title.
   Bardivisions=bardivisions, $ ; The barcolor divisions.
   Barminor=barminor, $         ; The barcolor minor ticks.
   Barformat=barformat, $       ; The barcolor format.
   Barminval=barminval, $       ; The barcolor min. value.
   Barmaxval=barmaxval, $       ; The barcolor max. value.
   Invertcolors=invertcolors, $ ; The invertcolors barcolor flag.
   Cb_Log=cb_log, $             ; The barcolor log scale option.
   Irregular=irregular, $       ; Irregular grids contour.
   Isotropic=isotropic, $       ; Isotropic grids contour.
   Downhill=downhill, $         ; Downhill grids contour.
   NoAxisInfo=noaxisinfo, $     ; Set this keyword to inhibit axis information on the GUI.
   XAxis=xaxis, $               ; An IAA_AXIS object for the X axis.
   YAxis=yaxis, $               ; An IAA_AXIS object for the Y axis.
   _Extra=extra                 ; Extra keywords to be passed to IAA_AXIS objects.

; This method performs the initialization of the object.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_ContourConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

   ; Check for keywords and parameters.

IF N_Elements(nlevels) EQ 0 THEN nlevels = 15
IF N_Elements(charsize) EQ 0 THEN charsize = 1.0
IF N_Elements(charthick) EQ 0 THEN charthick = 1.0
IF N_Elements(font) EQ 0 THEN font = !P.Font
IF N_Elements(position) EQ 0 THEN position = [0.10, 0.15, 0.80, 0.95]
IF N_Elements(subtitle) EQ 0 THEN subtitle = ''
IF N_Elements(title) EQ 0 THEN title = ''
IF N_Elements(c_linestyle) EQ 0 THEN c_linestyle = 0
IF N_Elements(thick) EQ 0 THEN thick = 1.0
IF N_Elements(ticklen) EQ 0 THEN ticklen = 0.02
IF N_Elements(fill) EQ 0 THEN fill = 0
IF N_Elements(follow) EQ 0 THEN follow = 0
IF N_Elements(overplot) EQ 0 THEN overplot = 0
IF N_Elements(noerase) EQ 0 THEN noerase = 0

IF N_Elements(bartitle) EQ 0 THEN bartitle = ''
IF N_Elements(bardivisions) EQ 0 THEN bardivisions = 6
IF N_Elements(barminor) EQ 0 THEN barminor = 2
IF N_Elements(barformat) EQ 0 THEN barformat = '(I5)'
IF N_Elements(barminval) EQ 0 THEN barminval = 0
IF N_Elements(barmaxval) EQ 0 THEN barmaxval = 0
IF N_Elements(invertcolors) EQ 0 THEN invertcolors = 0
IF N_Elements(cb_log) EQ 0 THEN cb_log = 0

IF N_Elements(irregular) EQ 0 THEN irregular = 0
IF N_Elements(isotropic) EQ 0 THEN isotropic = 0
IF N_Elements(downhill) EQ 0 THEN downhill = 0

   ; Use different colors in 8-bit environment.

Device, Get_Visual_Depth=theDepth
IF theDepth GT 8 THEN BEGIN
   IF N_Elements(background) EQ 0 THEN background = 'IVORY'
   IF N_Elements(color) EQ 0 THEN color = 'SADDLE BROWN'
ENDIF ELSE BEGIN
   IF N_Elements(background) EQ 0 THEN background = 'GRAY'
   IF N_Elements(color) EQ 0 THEN color = 'GREEN'
ENDELSE

   ; Populate the object.

self.nlevels = nlevels
self.background = background
self.charsize = charsize
self.charthick = charthick
self.color = color
self.font = font
self.position = position
self.subtitle = subtitle
self.title = title
self.c_linestyle = c_linestyle
self.thick = thick
self.ticklen = ticklen
self.fill = fill
self.follow = follow
self.noerase = noerase

self.bartitle = bartitle
self.bardivisions = bardivisions
self.barminor = barminor
self.barformat = barformat
self.barminval = barminval
self.barmaxval = barmaxval
self.invertcolors = invertcolors
self.cb_log = cb_log

self.irregular = irregular
self.isotropic = isotropic
self.downhill = downhill

self.noaxisinfo = Keyword_Set(noaxisinfo)

   ; Do we have to create the axis objects ourself?

IF Obj_Valid(xaxis) THEN BEGIN
   self.xaxis = xaxis
   self.createaxis = 0
ENDIF ELSE BEGIN
   self.xaxis = Obj_New('IAA_Axis', /XAxis, _Extra=extra)
   self.createaxis = 1
ENDELSE

IF Obj_Valid(yaxis) THEN BEGIN
   self.yaxis = yaxis
   self.createaxis = 0
ENDIF ELSE BEGIN
   self.yaxis = Obj_New('IAA_Axis', /YAxis, _Extra=extra)
   self.createaxis = 1
ENDELSE

RETURN, 1
END ;---------------------------------------------------------------------------------------------------



FUNCTION IAA_ContourConfig_GetValue, id

; The is the function that returns the "value" of the compound widget.
; In this case, the "value" is the current keyword structure.

child = Widget_Info(id, /Child)
Widget_Control, child, Get_UValue=self
self->UpdateObject
stateStruct = self->GetKeywords()
RETURN, stateStruct
END ;-----------------------------------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig_SetValue, value

; This is the procedure that sets the "value" of the compound widget.
; At the moment, I don't know what it would mean to "set" the value,
; so I am leaving this unimplemented.

ok = Dialog_Message('Setting the value of the compound widget is undefined.')
END ;-----------------------------------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig_Widget_Events, event

; The main event handler for the compound widget. It reacts
; to "messages" in the UValue of the text widget.
; The message indicates which object method to call. A message
; consists of an object method and the self object reference.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = Dialog_Message('Specified functionality currently  NOT IMPLEMENTED.')
   RETURN
ENDIF

Widget_Control, event.ID, Get_UValue=theMessage
event = Call_Method(theMessage.method, theMessage.object, event)

END ;-----------------------------------------------------------------------------------------------------------------------------




PRO IAA_ContourConfig::Cleanup

; The cleanup method.

Obj_Destroy, self.colorObj
Obj_Destroy, self.bcolorObj

IF self.createaxis THEN BEGIN
   Obj_Destroy, self.xaxis
   Obj_Destroy, self.yaxis
ENDIF

END ;---------------------------------------------------------------------------------------------------



PRO IAA_ContourConfig__Define

struct = { IAA_CONTOURCONFIG, $         ; The IAA_CONTOURCONFIG object class.

   ; Properties of the CONTOUR command.

           nlevels: 0, $             ; The number of levels
           background: '', $         ; The name of the background color.
           charsize: 0.0, $          ; The contour charsize.
           charthick: 0.0, $         ; The thickness of the characters.
           color: '', $              ; The name of the color.
           font: 0, $                ; The contour font flag.
           position: FltArr(4), $    ; The position in the contour window.
           subtitle: '', $           ; The contour subtitle.
           title: '', $              ; The contour title.
           c_linestyle: 0, $         ; The contour linestyle.
           thick: 0.0, $             ; The contour line thickness.
           ticklen: 0.0, $           ; The contour tick length.
           fill: 0, $                ; The contour fill value.
           follow: 0, $              ; The contour follow value.
           overplot: 0, $            ; The contour overplot value.
           noerase: 0, $             ; The contour noerase value.
           event_pro: '', $          ; The external event procedure.
           event_func: '', $         ; The external event function.

           irregular: 0, $           ; Irregular grids contour.
           isotropic: 0, $           ; Isotropic grids contour.
           downhill: 0, $            ; Downhill grids contour.

   ; Properties of the COLORBAR.

           bartitle: '', $           ; The barcolor title.
           bardivisions: 0, $        ; The number of divisions
           barminor: 0, $            ; The number of minor ticks
           barformat: '', $          ; The format to use with tick marks. May be name of procedure.
           barminval: 0.0, $         ; The colorbar min.value.
           barmaxval: 0.0, $         ; The colorbar max.value.
           invertcolors: 0, $        ; The invertcolors barcolor flag.
           cb_log: 0, $              ; The colorbar log scale option.

   ; Widget identifiers.

           tlb: 0L, $                 ; The top-level base widget identifier.
           titleObj: Obj_New(), $     ; The contour title field object identifier.
           subTitleObj: Obj_New(), $  ; The contour subtitle field object identifier.
           linestyleID: Obj_New(), $  ; The line style droplist object identifier.
           thickObj: Obj_New(), $     ; The line thickness field object identifier.
           ticklenObj: Obj_New(), $   ; The tick length field object identifier.
           charthickObj: Obj_New(), $ ; The character thickness field object identifier.
           charsizeObj: Obj_New(), $  ; The character size field object identifier.
           colorObj: Obj_New(), $     ; The contour color object identifier.
           bcolorObj: Obj_New(), $    ; The contour background color object identifier.
           nlevelsObj: Obj_New(), $   ; The number of levels object identifier.
           bartitleObj: Obj_New(), $  ; The barcolor title field object identifier.
           bardivisionsObj: Obj_New(), $   ; The barcolor divisions object identifier.
           barminorObj: Obj_New(), $  ; The barcolor minor ticks object identifier.
           barformatObj: Obj_New(), $ ; The barcolor format object identifier.
           barminvalObj: Obj_New(), $ ; The barcolor min. value object identifier.
           barmaxvalObj: Obj_New(), $ ; The barcolor max. value object identifier.

   ; Other information.

           createaxis: 0L, $        ; A flag that indicates this program created the axis objects.
           lastvalidline: 0L, $     ; The index of the last valid line style.
           xaxis: Obj_New(), $      ; The X AXIS object of class IAA_AXIS.
           yaxis: Obj_New(), $      ; The Y AXIS object of class IAA_AXIS.
           labelfont: '', $         ; The font name for the program labels.
           all_events: 0L, $        ; A flag that sends all events to the specified event handler.
           noaxisinfo: 0L, $        ; A flag to prevent axis information from being displayed or presented.
           notifyID: LonArr(2)  $   ; The widget ID and top-level base of the widget to notify.
           }
END ;---------------------------------------------------------------------------------------------------

