;+
; NAME:
;       IAA_OVERPLOTCONFIG__DEFINE
;
; AUTHOR:
;
;       Jos Enrique Ruiz
;       Inspired upon David Fanning's MPI_PLOTCONFIG__DEFINE
;
;
; CATEGORY:
;
;       Graphics
;
; CALLING SEQUENCE:
;
;       overplotConfigObj = Obj_New("IAA_OVERPLOTCONFIG")
;
; INPUT PARAMETERS:
;
;       None.
;
; INPUT KEYWORDS (Sent to the INIT method. The same keywords can be set with the SETPROPERTY method of the object.):
;
;       COLOR - The name of the data color.
;
;       LINESTYLE - The overplot linestyle. By default, 0. Possible values are:
;         0 - Solid Line
;         1 - Dotted
;         2 - Dashed
;         3 - Dash Dot
;         4 - Dash Dot Dot
;         5 - Long Dash
;         6 - No Line
;
;       PSYM - The overplot symbol value. By default, 18. Possible values are:
;         0 - Dot
;         1 - Filled Circle
;         2 - Filled Upward Triangle
;         3 - Filled Downward Triangle
;         4 - Filled Diamond
;         5 - Filled Square
;         6 - Open Circle
;         7 - Open Upward Triangle
;         8 - Open Downward Triangle
;         9 - Open Diamond
;        10 - Open Square
;        11 - Plus Sign
;        12 - X
;        13 - Star
;        14 - Filed Rightfacing Triangle
;        15 - Filled Leftfacing Triangle
;        16 - Open Rightfacing Triangle
;        17 - Open Leftfacing Triangle
;        18 - No Symbol (the default).
;
;      SYMSIZE - The overplot symbol size. By default, 1.0.
;
;      THICK - The overplot line thickness. By default, 1.0.
;
; METHOD PROCEDURES:
;
;      GUI - This procedure method displays a graphical user interface that allows the user
;            to change overplot configuration parameters. The following keywords can be used:
;
;            GROUP_LEADER - The group leader for this GUI. If this keyword is used, the program will be
;               distroyed when the group leader is destroyed.
;
;            NOTIFYID - A two-element array containing the widget identifier and top-level base ID of a widget
;               designated to receive an event from this program. The event structure will be defined and sent
;               like this:
;
;                    Widget_Control, notifyid[0], Send_Event={ IAA_OVERPLOTCONFIG_EVENT, $
;                                                              ID: notifyid[0], $
;                                                              TOP:notifyid[1], $
;                                                              HANDLER: 0L, $
;                                                              OBJECT: self }
;
;
; PROGRAM NOTES:
;
;      Color Names: Color names are those used with FSC_Color and PickColorName. See the
;         documentation for those programs for instuctions on loading your own colors.
;         To see the default colors and names, type this:
;
;                IDL> color = PickColorName('yellow')
;
;


PRO IAA_OverPlotConfig_CenterTLB, tlb

; Center the top-level base on the display.

screenSize = Get_Screen_Size()
IF screenSize[0] GT 2000 THEN screenSize[0] = screenSize[0]/2 ; Dual monitors.
xCenter = screenSize(0) / 2
yCenter = screenSize(1) / 2

geom = Widget_Info(tlb, /Geometry)
xHalfSize = geom.Scr_XSize / 2
yHalfSize = geom.Scr_YSize / 2

Widget_Control, tlb, XOffset = xCenter-xHalfSize, $
   YOffset = yCenter-yHalfSize

END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_OverPlotConfig_Error_Message, theMessage, Traceback=traceback, NoName=noName, _Extra=extra

On_Error, 2

   ; Check for presence and type of message.

IF N_Elements(theMessage) EQ 0 THEN theMessage = !Error_State.Msg
s = Size(theMessage)
messageType = s[s[0]+1]
IF messageType NE 7 THEN BEGIN
   Message, "The message parameter must be a string.", _Extra=extra
ENDIF

   ; Get the call stack and the calling routine's name.

Help, Calls=callStack
callingRoutine = (StrSplit(StrCompress(callStack[1])," ", /Extract))[0]


   ; Are widgets supported? Doesn't matter in IDL 5.3 and higher.

widgetsSupported = ((!D.Flags AND 65536L) NE 0) OR Float(!Version.Release) GE 5.3
IF widgetsSupported THEN BEGIN
   IF Keyword_Set(noName) THEN answer = Dialog_Message(theMessage, _Extra=extra) ELSE BEGIN
      IF StrUpCase(callingRoutine) EQ "$MAIN$" THEN answer = Dialog_Message(theMessage, _Extra=extra) ELSE $
         answer = Dialog_Message(StrUpCase(callingRoutine) + ": " + theMessage, _Extra=extra)
   ENDELSE
ENDIF ELSE BEGIN
      Message, theMessage, /Continue, /NoPrint, /NoName, /NoPrefix, _Extra=extra
      Print, '%' + callingRoutine + ': ' + theMessage
      answer = 'OK'
ENDELSE

   ; Provide traceback information if requested.

IF Keyword_Set(traceback) THEN BEGIN
   Help, /Last_Message, Output=traceback
   Print,''
   Print, 'Traceback Report from ' + StrUpCase(callingRoutine) + ':'
   Print, ''
   FOR j=0,N_Elements(traceback)-1 DO Print, "     " + traceback[j]
ENDIF

RETURN, answer
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_OverPlotConfig::Symbol, number

   ; This method calculates a user-defined symbol for the PSYM keyword.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 18
ENDIF

   ; Error checking.

IF N_Elements(number) EQ 0 THEN RETURN, 18
IF (number LT 0) OR (number GT 18) THEN Message, /NoName, 'Symbol number out of defined range. Returning NO SYMBOL.'

   ; Define helper variables for creating circles.

phi = Findgen(32) * (!PI * 2 / 32.)
phi = [ phi, phi(0) ]

   ; Use user defined symbol by default.

result = 8

CASE number OF

    0  : result = 3                                                ; Dot
    1  : UserSym, Cos(phi), Sin(phi), /Fill                        ; Filled circle.
    2  : UserSym, [ -1, 0, 1, -1 ], [ -1, 1, -1, -1 ], /Fill       ; Filled upward triangle.
    3  : UserSym, [ -1, 0, 1, -1 ], [  1, -1, 1, 1 ], /Fill        ; Filled downward triangle.
    4  : UserSym, [ 0, 1, 0, -1, 0 ], [ 1, 0, -1, 0, 1 ], /Fill    ; Filled diamond.
    5  : UserSym, [ -1, 1, 1, -1, -1 ], [ 1, 1, -1, -1, 1 ], /Fill ; Filled square.
    6  : UserSym, cos(phi), sin(phi)                               ; Open circle.
    7  : UserSym, [ -1, 0, 1, -1 ], [ -1, 1, -1, -1 ]              ; Open upward triangle.
    8  : UserSym, [ -1, 0, 1, -1 ], [  1, -1, 1, 1 ]               ; Open downward triangle.
    9  : UserSym, [ 0, 1, 0, -1, 0 ], [ 1, 0, -1, 0, 1 ]           ; Open diamond.
   10  : UserSym, [ -1, 1, 1, -1, -1 ], [ 1, 1, -1, -1, 1 ]        ; Open square.
   11  : result = 1                                                ; Plus.
   12  : result = 7                                                ; X.
   13  : result = 2                                                ; Star.
   14  : UserSym, [ -1, 1, -1, -1 ], [1, 0, -1, 1 ], /Fill         ; Filled rightfacing triangle.
   15  : UserSym, [ 1, -1, 1, 1 ], [1, 0, -1, 1 ], /Fill           ; Filled leftfacing triangle.
   16  : UserSym, [ -1, 1, -1, -1 ], [1, 0, -1, 1 ]                ; Open rightfacing triangle.
   17  : UserSym, [ 1, -1, 1, 1 ], [1, 0, -1, 1 ]                  ; Open leftfacing triangle.
   18  : result  = 0                                               ; No symbol at all.

ENDCASE

RETURN, result
END ;-----------------------------------------------------------------------------------------------------------------------------


PRO IAA_OverPlotConfig::Destroy

; This method destroys the top-level base of the compound widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN
ENDIF

IF Widget_Info(self.tlb, /Valid_ID) THEN Widget_Control, self.tlb, /Destroy
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_OverPlotConfig::GetTLB

; This method returns the top-level base of the compound widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

RETURN, self.tlb
END ;-----------------------------------------------------------------------------------------------------------------------------

PRO IAA_OverPlotConfig::SendEvent

; This method sends an event to the widget to be notified, if one exists.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN
ENDIF

IF Total(self.notifyID) GT 0 THEN BEGIN

   thisEvent = { IAA_OVERPLOTCONFIG_EVENT, $
                 ID: self.notifyID[0], $
                 TOP:self.notifyID[1], $
                 HANDLER: 0L, $
                 NAME:'OVERPLOT', $
                 OBJECT: self }

   Widget_Control, self.notifyID[0], Send_Event=thisEvent
ENDIF

END ;-----------------------------------------------------------------------------------------------------------------------------

PRO IAA_OverPlotConfig::UpdateObject

; This method simply gathers all the information in the graphical
; user interface (that doesn't update itself) and updates the self
; object. If fields are undefined, they are set to their default values.


IF Obj_Valid(self.thickObj) NE 0 THEN BEGIN
   IF Finite(self.thickObj->Get_Value()) EQ 0 THEN BEGIN
      value = 1.0
      self.thickObj->Set_Value, value
   ENDIF ELSE value = self.thickObj->Get_Value()
self.thick = value
ENDIF

IF Obj_Valid(self.symsizeObj) NE 0 THEN BEGIN
   IF Finite(self.symsizeObj->Get_Value()) EQ 0 THEN BEGIN
      value = 1.0
      self.symsizeObj->Set_Value, value
   ENDIF ELSE value = self.symsizeObj->Get_Value()
self.symsize = value
ENDIF



END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_OverPlotConfig::CheckEvent, event

; This event handler method checks to see if an event
; should be passed on.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

   ; Make sure all values are current by updating.

self->UpdateObject
self->SendEvent

RETURN, 0
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_OverPlotConfig::SetList, event

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

structArr = *self.ptroplot

self.oplotind = event.index

self.color = structArr[event.index].color
self.dcolorObj->SetColor, self.color
self.dcolorObj->Realize

self.linestyle = structArr[event.index].linestyle
self.lineStyleID->SetIndex, self.linestyle

self.psym = structArr[event.index].psym
self.psymID->SetIndex, 0 > Abs(self.psym) < 19

self.thick = structArr[event.index].thick
self.thickObj->Set_Value, self.thick

self.symsize = structArr[event.index].symsize
self.symsizeObj->Set_Value, self.symsize

RETURN, 1

END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_OverPlotConfig::SetLinestyle, event

; This event handler method sets the overplot line style.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

   ; Can't have no symbols and no lines. If this is the
   ; case, set the symbol droplist to dots.

IF self.lineStyleID->GetSelection() EQ 'No Line' THEN BEGIN
   IF self.psymID->GetSelection() EQ 'No Symbol' THEN BEGIN
      self.psym = self.lastvalidsymbol
      self.psymID->SetIndex, self.lastvalidsymbol
      self.noline = 1
   ENDIF
   self.noline = 1
ENDIF ELSE BEGIN
   self.lastvalidline = event.index
   self.noline = -1
ENDELSE

self.linestyle = event.index

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------



FUNCTION IAA_OverPlotConfig::SetSymbol, event

; This event handler method sets the overplot symbol type.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

   ; Can't have no symbols and no lines. If this is the
   ; case, set the line style droplist to solid lines.

IF self.psymID->GetSelection() EQ 'No Symbol' THEN BEGIN
   IF self.lineStyleID->GetSelection() EQ 'No Line' THEN BEGIN
      self.lineStyleID->SetIndex, self.lastvalidline
      self.linestyle = self.lastvalidline
      self.noline = 1
   ENDIF
ENDIF ELSE self.lastvalidsymbol = event.index

self.psym = event.index

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_OverPlotConfig::SetColor, event

; This event handler method sets the axis color.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.color = event.color

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_OverPlotConfig::SetLineThick, event

; This event handler method sets the overplot line thickness.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.thick = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_OverPlotConfig::SetSymSize, event

; This event handler method sets the overplot symbol size.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self.symsize = *event.value

RETURN, self->CheckEvent(event)
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_OverPlotConfig::DismissDialog, event

; This event handler method destroys the widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

Widget_Control, self.tlb, /Destroy

RETURN, 0
END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_OverPlotConfig::ApplyDialog, event

; This event handler method updates the dialog and destroys the widget.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self->UpdateObject
self->SendEvent

RETURN, 0
END ;-----------------------------------------------------------------------------------------------------------------------------

FUNCTION IAA_OverPlotConfig::GetKeywords

; This method constructs a structure containing fields with values
; set to the current state of the object.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

self->UpdateObject
struct = Create_Struct( $
   'oplotind', self.oplotind, $
   'color', self.color, $
   'linestyle', self.linestyle, $
   'psym', self.psym * self.noline, $
   'symsize', self.symsize, $
   'thick', self.thick, $
   'noline', self.noline $
    )

RETURN, struct
END ;-----------------------------------------------------------------------------------------------------------------------------

PRO IAA_OverPlotConfig::GUI, $

; This method displays a graphical user interface for changing the values in the object.

   Group_Leader=group_leader, $         ; The group leader for this GUI.
   NotifyID=notifyID, $                 ; The widget ID and top-level base to notify upon an event.
   OverPlotArr=overplotarr              ; Oveplots properties array.

   ; Error handling.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN
ENDIF

   ; Only one version of this program at a time.

IF XRegistered('iaa_overplotconfig') GT 0 THEN RETURN

   ; Check keyword arguments.

IF N_Elements(notifyID) NE 0 THEN self.notifyID = notifyID
structArr = *overplotarr
self.color = structArr[0].color
self.thick = structArr[0].thick
self.linestyle = structArr[0].linestyle
self.psym = structArr[0].psym
self.symsize = structArr[0].symsize

labeldefaultsize = 55

   ; Create the widget interface.

IF N_Elements(group_leader) EQ 0 THEN BEGIN
   tlb = Widget_Base(Column=1, Title='OverPlot Options...', $ ;Base_Align_Center=1
      Pro_Set_Value="IAA_OVERPLOT_SET_VALUE", Func_Get_Value="IAA_OVERPLOT_GET_VALUE", Map=0, /tlb_frame_attr)
ENDIF ELSE BEGIN
   tlb = Widget_Base(Column=1, Title='OverPlot Options...', $ ;Base_Align_Center=1, $
      Pro_Set_Value="IAA_OVERPLOT_SET_VALUE", Func_Get_Value="IAA_OVERPLOT_GET_VALUE", $
      Group_Leader=group_leader, Map=0, /tlb_frame_attr)
ENDELSE

mainbase = Widget_Base(tlb, Column=1, Frame=1, UValue=self, Base_Align_Center=1)

    oplotlist = Widget_Base(mainbase, Row=1, Base_Align_Center=1)

    listArr = Make_Array(N_Elements(structArr), /integer, /index)
    listID = FSC_Droplist(oplotlist, Value=listArr+1, Index=0, $
      UValue={method:'SetList', object:self}, Title='Overplot Number:', Space=[1,1])

   droprow = Widget_Base(mainbase, Row=1, Base_Align_Center=1)

   styles = ['Solid', 'Dotted', 'Dashed', 'Dash Dot', 'Dash Dot Dot', 'Long Dash', 'No Line']
   IF self.noline EQ 1 THEN self.linestyle = 6
   linestyleID = FSC_Droplist(droprow, Value=styles, Index=self.linestyle, $
      UValue={method:'SetLineStyle', object:self}, Title='Line Style:', Space=[1,1])

   symbols = ['Dot', 'Filled Circle', 'Filled Upward Triangle', 'Filled Downward Triangle', $
              'Filled Diamond', 'Filled Square', 'Open Circle', 'Open Upward Triangle', $
              'Open Downward Triangle', 'Open Diamond', 'Open Square', 'Plus Sign', 'X', $
              'Star', 'Filled Right Triangle', 'Filled Left Triangle', 'Open Right Triangle', $
              'Open Left Triangle', 'No Symbol']

   psymID = FSC_Droplist(droprow, Value=symbols, Index=0 > Abs(self.psym) < 19, $
      UValue={method:'SetSymbol', object:self}, Title='Symbol:', Space=[1,1])


   row = Widget_Base(mainbase, Row=1)
   column1 = Widget_Base(row, Column=1)
   column2 = Widget_Base(row, Column=1)
   column3 = Widget_Base(row, Column=1)

   thickID = FSC_Field(column1, Value=Float(self.thick), Title='Line Thickness: ', $
      Label_Left=1, Object=thickObj, XSize=6, Decimal=3, $
      UValue={method:'SETLINETHICK', object:self}, LabelSize=labeldefaultsize*2, $   ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_OverPlotConfig_Widget_Events")

   symsizeID = FSC_Field(column2, Value=Float(self.symsize), Title='Symbol Size: ', $
      Label_Left=1, Object=symsizeObj, XSize=6, Decimal=3, $
      UValue={method:'SETSYMSIZE', object:self}, LabelSize=labeldefaultsize+25, $    ; ++mgs/--mgs
      /Cr_Only, Event_Pro="IAA_OverPlotConfig_Widget_Events")

   acolorID = CW_Drawcolor(column3, Color=self.color, LabelText='   Data Color: ', Title='Data Color', $
         UValue={method:'SETCOLOR', object:self}, Label_Left=1, Object=dcolorObj)

  buttonbase = Widget_Base(tlb, Row=1, Align_Center=1)
  button = Widget_Button(buttonbase, Value='Close', UValue={Object:self, Method:'DISMISSDIALOG'})
  button = Widget_Button(buttonbase, Value='Apply', UValue={Object:self, Method:'APPLYDIALOG'})


IAA_OverPlotConfig_CenterTLB, tlb

Widget_Control, tlb, /Realize

   ; Fill out object fields.

self.tlb = tlb
self.thickObj = thickObj
self.symsizeObj = symsizeObj
self.dcolorObj = dcolorObj
self.lineStyleID = linestyleID
self.psymID = psymID
self.listID = listID
self.ptroplot = overplotarr

Widget_Control, tlb, Map=1
XManager, 'iaa_overplotconfig', tlb, Event_Handler='IAA_OverPlotConfig_Widget_Events', /No_Block

END ;-----------------------------------------------------------------------------------------------------------------------------


FUNCTION IAA_OverPlotConfig::Init, $

; This is the initialization method for the IAA_OVERPLOTCONFIG object.

   Color=color, $       ; The name of the data color.
   Linestyle=linestyle, $       ; The overplot linestyle.
   PSym=psym, $                 ; The overplot symbol value.
   Symsize=symsize, $           ; The overplot symbol size.
   Thick=thick                  ; The overplot line thickness.

; This method performs the initialization of the object.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = IAA_OverPlotConfig_Error_Message(/Traceback)
   RETURN, 0
ENDIF

   ; Check for keywords and parameters.

IF N_Elements(linestyle) EQ 0 THEN linestyle = 0
IF N_Elements(psym) EQ 0 THEN psym = 18
IF N_Elements(symsize) EQ 0 THEN symsize = 1.0
IF N_Elements(thick) EQ 0 THEN thick = 1.0

   ; Use different colors in 8-bit environment.

Device, Get_Visual_Depth=theDepth
IF theDepth GT 8 THEN BEGIN
   IF N_Elements(color) EQ 0 THEN color = 'NAVY'
ENDIF ELSE BEGIN
   IF N_Elements(color) EQ 0 THEN color = 'YELLOW'
ENDELSE

   ; Populate the object.

self.color = color
self.linestyle = linestyle
self.psym = psym
IF self.psym LT 0 THEN BEGIN
   self.noline = -1
   self.psym = Abs(self.psym)
ENDIF ELSE IF self.psym EQ 18 THEN self.noline = -1 ELSE self.noline = 1
self.symsize = symsize
self.thick = thick

RETURN, 1
END ;---------------------------------------------------------------------------------------------------

PRO IAA_OverPlotConfig_Widget_Events, event

; The main event handler for the compound widget. It reacts
; to "messages" in the UValue of the text widget.
; The message indicates which object method to call. A message
; consists of an object method and the self object reference.

Catch, theError
IF theError NE 0 THEN BEGIN
   ok = Dialog_Message('Specified functionality currently  NOT IMPLEMENTED.')
   RETURN
ENDIF

Widget_Control, event.ID, Get_UValue=theMessage
event = Call_Method(theMessage.method, theMessage.object, event)

END ;-----------------------------------------------------------------------------------------------------------------------------

PRO IAA_OverPlotConfig::Cleanup

; The cleanup method.

Obj_Destroy, self.psymID
Obj_Destroy, self.linestyleID
Obj_Destroy, self.thickObj
Obj_Destroy, self.symsizeObj
Obj_Destroy, self.dcolorObj

Ptr_Free, self.ptroplot

END ;---------------------------------------------------------------------------------------------------



PRO IAA_OverPlotConfig__Define

struct = { IAA_OVERPLOTCONFIG, $     ; The IAA_OVERPLOTCONFIG object class.

           oplotind: 0L, $           ; The index of the overplot.
           color: '', $          ; The name of the axis color.
           linestyle: 0, $           ; The overplot linestyle.
           psym: 0L, $               ; The overplot symbol.
           symsize: 0.0, $           ; The overplot symbol size.
           thick: 0.0, $             ; The overplot line thickness.

   ; Widget identifiers.

            tlb: 0L, $                 ; The top-level base widget identifier.
            listID: Obj_New(), $       ; The overplot list object identifier.
            psymID: Obj_New(), $       ; The overplot symbol droplist object identifier.
            linestyleID: Obj_New(), $  ; The line style droplist object identifier.
            thickObj: Obj_New(), $     ; The line thickness field object identifier.
            symsizeObj: Obj_New(), $   ; The overplot symbol field object identifier.
            dcolorObj: Obj_New(), $    ; The overplot data color object identifier.

            ptroplot: Ptr_New(), $      ; The pointer to overplot array info.

   ; Other information.

            lastvalidsymbol: 0L, $   ; The index of the last valid symbol.
            lastvalidline: 0L, $     ; The index of the last valid line style.
            noline: 0L, $            ; A flag to tell if symbols should be conntect by lines (-1) on not (1).
            notifyID: LonArr(2)  $   ; The widget ID and top-level base of the widget to notify.
           }
END ;---------------------------------------------------------------------------------------------------

